# flake8: noqa: F401
from .abc import (
    ABCIE,
    ABCIViewIE,
    ABCIViewShowSeriesIE,
)
from .abcnews import (
    AbcNewsIE,
    AbcNewsVideoIE,
)
from .abcotvs import (
    ABCOTVSIE,
    ABCOTVSClipsIE,
)
from .abematv import (
    AbemaTVIE,
    AbemaTVTitleIE,
)
from .academicearth import AcademicEarthCourseIE
from .acast import (
    ACastChannelIE,
    ACastIE,
)
from .acfun import (
    AcFunBangumiIE,
    AcFunVideoIE,
)
from .adn import (
    ADNIE,
    ADNSeasonIE,
)
from .adobeconnect import AdobeConnectIE
from .adobetv import AdobeTVVideoIE
from .adultswim import AdultSwimIE
from .aenetworks import (
    AENetworksCollectionIE,
    AENetworksIE,
    AENetworksShowIE,
    BiographyIE,
    HistoryPlayerIE,
    HistoryTopicIE,
)
from .aeonco import AeonCoIE
from .afreecatv import (
    AfreecaTVCatchStoryIE,
    AfreecaTVIE,
    AfreecaTVLiveIE,
    AfreecaTVUserIE,
)
from .agalega import AGalegaIE
from .agora import (
    TokFMAuditionIE,
    TokFMPodcastIE,
    WyborczaPodcastIE,
    WyborczaVideoIE,
)
from .aitube import AitubeKZVideoIE
from .alibaba import AlibabaIE
from .aliexpress import AliExpressLiveIE
from .aljazeera import AlJazeeraIE
from .allocine import AllocineIE
from .allstar import (
    AllstarIE,
    AllstarProfileIE,
)
from .alphaporno import AlphaPornoIE
from .altcensored import (
    AltCensoredChannelIE,
    AltCensoredIE,
)
from .alura import (
    AluraCourseIE,
    AluraIE,
)
from .amadeustv import AmadeusTVIE
from .amara import AmaraIE
from .amazon import (
    AmazonReviewsIE,
    AmazonStoreIE,
)
from .amazonminitv import (
    AmazonMiniTVIE,
    AmazonMiniTVSeasonIE,
    AmazonMiniTVSeriesIE,
)
from .amcnetworks import AMCNetworksIE
from .americastestkitchen import (
    AmericasTestKitchenIE,
    AmericasTestKitchenSeasonIE,
)
from .angel import AngelIE
from .antenna import (
    Ant1NewsGrArticleIE,
    Ant1NewsGrEmbedIE,
    AntennaGrWatchIE,
)
from .anvato import AnvatoIE
from .aol import AolIE
from .apa import APAIE
from .aparat import AparatIE
from .appleconnect import AppleConnectIE
from .applepodcasts import ApplePodcastsIE
from .archiveorg import (
    ArchiveOrgIE,
    YoutubeWebArchiveIE,
)
from .arcpublishing import ArcPublishingIE
from .ard import (
    ARDIE,
    ARDAudiothekIE,
    ARDAudiothekPlaylistIE,
    ARDBetaMediathekIE,
    ARDMediathekCollectionIE,
)
from .arnes import ArnesIE
from .art19 import (
    Art19IE,
    Art19ShowIE,
)
from .arte import (
    ArteTVCategoryIE,
    ArteTVEmbedIE,
    ArteTVIE,
    ArteTVPlaylistIE,
)
from .asobichannel import (
    AsobiChannelIE,
    AsobiChannelTagURLIE,
)
from .asobistage import AsobiStageIE
from .atresplayer import AtresPlayerIE
from .atscaleconf import AtScaleConfEventIE
from .audimedia import AudiMediaIE
from .audioboom import AudioBoomIE
from .audiodraft import (
    AudiodraftCustomIE,
    AudiodraftGenericIE,
)
from .audiomack import (
    AudiomackAlbumIE,
    AudiomackIE,
)
from .audius import (
    AudiusIE,
    AudiusPlaylistIE,
    AudiusProfileIE,
    AudiusTrackIE,
)
from .azmedien import AZMedienIE
from .baidu import BaiduVideoIE
from .banbye import (
    BanByeChannelIE,
    BanByeIE,
)
from .bandcamp import (
    BandcampAlbumIE,
    BandcampIE,
    BandcampUserIE,
    BandcampWeeklyIE,
)
from .bandlab import (
    BandlabIE,
    BandlabPlaylistIE,
)
from .bannedvideo import BannedVideoIE
from .bbc import (
    BBCIE,
    BBCCoUkArticleIE,
    BBCCoUkIE,
    BBCCoUkIPlayerEpisodesIE,
    BBCCoUkIPlayerGroupIE,
    BBCCoUkPlaylistIE,
)
from .beacon import BeaconTvIE
from .beatport import BeatportIE
from .beeg import BeegIE
from .behindkink import BehindKinkIE
from .berufetv import BerufeTVIE
from .bet import BetIE
from .bfi import BFIPlayerIE
from .bfmtv import (
    BFMTVIE,
    BFMTVArticleIE,
    BFMTVLiveIE,
)
from .bibeltv import (
    BibelTVLiveIE,
    BibelTVSeriesIE,
    BibelTVVideoIE,
)
from .bigo import BigoIE
from .bild import BildIE
from .bilibili import (
    BilibiliAudioAlbumIE,
    BilibiliAudioIE,
    BiliBiliBangumiIE,
    BiliBiliBangumiMediaIE,
    BiliBiliBangumiSeasonIE,
    BilibiliCategoryIE,
    BilibiliCheeseIE,
    BilibiliCheeseSeasonIE,
    BilibiliCollectionListIE,
    BiliBiliDynamicIE,
    BilibiliFavoritesListIE,
    BiliBiliIE,
    BiliBiliPlayerIE,
    BilibiliPlaylistIE,
    BiliBiliSearchIE,
    BilibiliSeriesListIE,
    BilibiliSpaceAudioIE,
    BilibiliSpaceVideoIE,
    BilibiliWatchlaterIE,
    BiliIntlIE,
    BiliIntlSeriesIE,
    BiliLiveIE,
)
from .biobiochiletv import BioBioChileTVIE
from .bitchute import (
    BitChuteChannelIE,
    BitChuteIE,
)
from .bitmovin import BitmovinIE
from .blackboardcollaborate import (
    BlackboardCollaborateIE,
    BlackboardCollaborateLaunchIE,
)
from .bleacherreport import (
    BleacherReportCMSIE,
    BleacherReportIE,
)
from .blerp import BlerpIE
from .blogger import BloggerIE
from .bloomberg import BloombergIE
from .bluesky import BlueskyIE
from .bongacams import BongaCamsIE
from .boosty import BoostyIE
from .bostonglobe import BostonGlobeIE
from .box import BoxIE
from .boxcast import BoxCastVideoIE
from .bpb import BpbIE
from .br import BRIE
from .brainpop import (
    BrainPOPELLIE,
    BrainPOPEspIE,
    BrainPOPFrIE,
    BrainPOPIE,
    BrainPOPIlIE,
    BrainPOPJrIE,
)
from .breitbart import BreitBartIE
from .brightcove import (
    BrightcoveLegacyIE,
    BrightcoveNewIE,
)
from .brilliantpala import (
    BrilliantpalaClassesIE,
    BrilliantpalaElearnIE,
)
from .btvplus import BTVPlusIE
from .bundesliga import BundesligaIE
from .bundestag import BundestagIE
from .bunnycdn import BunnyCdnIE
from .businessinsider import BusinessInsiderIE
from .buzzfeed import BuzzFeedIE
from .byutv import BYUtvIE
from .c56 import C56IE
from .caltrans import CaltransIE
from .cam4 import CAM4IE
from .camfm import (
    CamFMEpisodeIE,
    CamFMShowIE,
)
from .cammodels import CamModelsIE
from .camsoda import CamsodaIE
from .camtasia import CamtasiaEmbedIE
from .canal1 import Canal1IE
from .canalalpha import CanalAlphaIE
from .canalc2 import Canalc2IE
from .canalplus import CanalplusIE
from .canalsurmas import CanalsurmasIE
from .caracoltv import CaracolTvPlayIE
from .cbc import (
    CBCIE,
    CBCGemContentIE,
    CBCGemIE,
    CBCGemLiveIE,
    CBCGemOlympicsIE,
    CBCGemPlaylistIE,
    CBCListenIE,
    CBCPlayerIE,
    CBCPlayerPlaylistIE,
)
from .cbs import (
    CBSIE,
    ParamountPressExpressIE,
)
from .cbsnews import (
    CBSLocalArticleIE,
    CBSLocalIE,
    CBSLocalLiveIE,
    CBSNewsEmbedIE,
    CBSNewsIE,
    CBSNewsLiveIE,
    CBSNewsLiveVideoIE,
)
from .cbssports import (
    CBSSportsEmbedIE,
    CBSSportsIE,
    TwentyFourSevenSportsIE,
)
from .ccc import (
    CCCIE,
    CCCPlaylistIE,
)
from .ccma import CCMAIE
from .cctv import CCTVIE
from .cda import (
    CDAIE,
    CDAFolderIE,
)
from .cellebrite import CellebriteIE
from .ceskatelevize import CeskaTelevizeIE
from .cgtn import CGTNIE
from .charlierose import CharlieRoseIE
from .chaturbate import ChaturbateIE
from .chilloutzone import ChilloutzoneIE
from .chzzk import (
    CHZZKLiveIE,
    CHZZKVideoIE,
)
from .cinemax import CinemaxIE
from .cinetecamilano import CinetecaMilanoIE
from .cineverse import (
    CineverseDetailsIE,
    CineverseIE,
)
from .ciscolive import (
    CiscoLiveSearchIE,
    CiscoLiveSessionIE,
)
from .ciscowebex import CiscoWebexIE
from .cjsw import CJSWIE
from .clipchamp import ClipchampIE
from .cliprs import ClipRsIE
from .closertotruth import CloserToTruthIE
from .cloudflarestream import CloudflareStreamIE
from .cloudycdn import CloudyCDNIE
from .clubic import ClubicIE
from .clyp import ClypIE
from .cnbc import CNBCVideoIE
from .cnn import (
    CNNIE,
    CNNIndonesiaIE,
)
from .comedycentral import ComedyCentralIE
from .commonmistakes import (
    BlobIE,
    CommonMistakesIE,
    UnicodeBOMIE,
)
from .commonprotocols import (
    MmsIE,
    RtmpIE,
    ViewSourceIE,
)
from .condenast import CondeNastIE
from .corus import CorusIE
from .coub import CoubIE
from .cozytv import CozyTVIE
from .cpac import (
    CPACIE,
    CPACPlaylistIE,
)
from .cracked import CrackedIE
from .craftsy import CraftsyIE
from .croatianfilm import CroatianFilmIE
from .crooksandliars import CrooksAndLiarsIE
from .crowdbunker import (
    CrowdBunkerChannelIE,
    CrowdBunkerIE,
)
from .crtvg import CrtvgIE
from .cspan import (
    CSpanCongressIE,
    CSpanIE,
)
from .ctsnews import CtsNewsIE
from .ctvnews import CTVNewsIE
from .cultureunplugged import CultureUnpluggedIE
from .curiositystream import (
    CuriosityStreamCollectionsIE,
    CuriosityStreamIE,
    CuriosityStreamSeriesIE,
)
from .cybrary import (
    CybraryCourseIE,
    CybraryIE,
)
from .dacast import (
    DacastPlaylistIE,
    DacastVODIE,
)
from .dailymail import DailyMailIE
from .dailymotion import (
    DailymotionIE,
    DailymotionPlaylistIE,
    DailymotionSearchIE,
    DailymotionUserIE,
)
from .dailywire import (
    DailyWireIE,
    DailyWirePodcastIE,
)
from .damtomo import (
    DamtomoRecordIE,
    DamtomoVideoIE,
)
from .dangalplay import (
    DangalPlayIE,
    DangalPlaySeasonIE,
)
from .daum import (
    DaumClipIE,
    DaumIE,
    DaumPlaylistIE,
    DaumUserIE,
)
from .daystar import DaystarClipIE
from .dbtv import DBTVIE
from .dctp import DctpTvIE
from .democracynow import DemocracynowIE
from .detik import DetikEmbedIE
from .deuxm import (
    DeuxMIE,
    DeuxMNewsIE,
)
from .dfb import DFBIE
from .dhm import DHMIE
from .digitalconcerthall import DigitalConcertHallIE
from .digiteka import DigitekaIE
from .digiview import DigiviewIE
from .discogs import DiscogsReleasePlaylistIE
from .disney import DisneyIE
from .dispeak import DigitallySpeakingIE
from .dlf import (
    DLFIE,
    DLFCorpusIE,
)
from .dlive import (
    DLiveStreamIE,
    DLiveVODIE,
)
from .douyutv import (
    DouyuShowIE,
    DouyuTVIE,
)
from .dplay import (
    TLCIE,
    AmHistoryChannelIE,
    AnimalPlanetIE,
    CookingChannelIE,
    DestinationAmericaIE,
    DiscoveryLifeIE,
    DiscoveryNetworksDeIE,
    DiscoveryPlusIE,
    DiscoveryPlusIndiaIE,
    DiscoveryPlusIndiaShowIE,
    DiscoveryPlusItalyIE,
    DiscoveryPlusItalyShowIE,
    DPlayIE,
    FoodNetworkIE,
    GoDiscoveryIE,
    HGTVDeIE,
    HGTVUsaIE,
    InvestigationDiscoveryIE,
    ScienceChannelIE,
    TravelChannelIE,
)
from .drbonanza import DRBonanzaIE
from .dreisat import DreiSatIE
from .dropbox import DropboxIE
from .dropout import (
    DropoutIE,
    DropoutSeasonIE,
)
from .drtalks import DrTalksIE
from .drtuber import DrTuberIE
from .drtv import (
    DRTVIE,
    DRTVLiveIE,
    DRTVSeasonIE,
    DRTVSeriesIE,
)
from .dtube import DTubeIE
from .dumpert import DumpertIE
from .duoplay import DuoplayIE
from .dvtv import DVTVIE
from .dw import (
    DWIE,
    DWArticleIE,
)
from .ebaumsworld import EbaumsWorldIE
from .ebay import EbayIE
from .egghead import (
    EggheadCourseIE,
    EggheadLessonIE,
)
from .eggs import (
    EggsArtistIE,
    EggsIE,
)
from .elementorembed import ElementorEmbedIE
from .elonet import ElonetIE
from .elpais import ElPaisIE
from .eltrecetv import ElTreceTVIE
from .embedly import EmbedlyIE
from .epicon import (
    EpiconIE,
    EpiconSeriesIE,
)
from .epidemicsound import EpidemicSoundIE
from .eplus import EplusIbIE
from .epoch import EpochIE
from .eporner import EpornerIE
from .erocast import ErocastIE
from .eroprofile import (
    EroProfileAlbumIE,
    EroProfileIE,
)
from .err import (
    ERRArhiivIE,
    ERRJupiterIE,
)
from .ertgr import (
    ERTFlixCodenameIE,
    ERTFlixIE,
    ERTWebtvEmbedIE,
)
from .espn import (
    ESPNIE,
    ESPNArticleIE,
    ESPNCricInfoIE,
    FiveThirtyEightIE,
    WatchESPNIE,
)
from .ettutv import EttuTvIE
from .europa import (
    EuropaIE,
    EuroParlWebstreamIE,
)
from .europeantour import EuropeanTourIE
from .eurosport import EurosportIE
from .euscreen import EUScreenIE
from .expressen import ExpressenIE
from .facebook import (
    FacebookAdsIE,
    FacebookIE,
    FacebookPluginsVideoIE,
    FacebookRedirectURLIE,
    FacebookReelIE,
)
from .fancode import (
    FancodeLiveIE,
    FancodeVodIE,
)
from .fathom import FathomIE
from .faulio import (
    FaulioIE,
    FaulioLiveIE,
)
from .faz import FazIE
from .fc2 import (
    FC2IE,
    FC2EmbedIE,
    FC2LiveIE,
)
from .fczenit import FczenitIE
from .fifa import FifaIE
from .filmarchiv import FilmArchivIE
from .filmon import (
    FilmOnChannelIE,
    FilmOnIE,
)
from .filmweb import FilmwebIE
from .firsttv import (
    FirstTVIE,
    FirstTVLiveIE,
)
from .fivetv import FiveTVIE
from .flextv import FlexTVIE
from .flickr import FlickrIE
from .floatplane import (
    FloatplaneChannelIE,
    FloatplaneIE,
)
from .folketinget import FolketingetIE
from .footyroom import FootyRoomIE
from .formula1 import Formula1IE
from .fourtube import (
    FourTubeIE,
    FuxIE,
    PornerBrosIE,
    PornTubeIE,
)
from .fox import FOXIE
from .fox9 import (
    FOX9IE,
    FOX9NewsIE,
)
from .foxnews import (
    FoxNewsArticleIE,
    FoxNewsIE,
    FoxNewsVideoIE,
)
from .foxsports import FoxSportsIE
from .fptplay import FptplayIE
from .francaisfacile import FrancaisFacileIE
from .francetv import (
    FranceTVIE,
    FranceTVInfoIE,
    FranceTVSiteIE,
)
from .freesound import FreesoundIE
from .freespeech import FreespeechIE
from .freetv import (
    FreeTvIE,
    FreeTvMoviesIE,
)
from .frontendmasters import (
    FrontendMastersCourseIE,
    FrontendMastersIE,
    FrontendMastersLessonIE,
)
from .funk import FunkIE
from .funker530 import Funker530IE
from .fuyintv import FuyinTVIE
from .gab import GabIE
from .gaia import GaiaIE
from .gamedevtv import GameDevTVDashboardIE
from .gamejolt import (
    GameJoltCommunityIE,
    GameJoltGameIE,
    GameJoltGameSoundtrackIE,
    GameJoltIE,
    GameJoltSearchIE,
    GameJoltUserIE,
)
from .gamespot import GameSpotIE
from .gamestar import GameStarIE
from .gaskrank import GaskrankIE
from .gazeta import GazetaIE
from .gbnews import GBNewsIE
from .gdcvault import GDCVaultIE
from .gedidigital import GediDigitalIE
from .generic import GenericIE
from .genericembeds import (
    HTML5MediaEmbedIE,
    QuotedHTMLIE,
)
from .genius import (
    GeniusIE,
    GeniusLyricsIE,
)
from .germanupa import GermanupaIE
from .getcourseru import (
    GetCourseRuIE,
    GetCourseRuPlayerIE,
)
from .gettr import (
    GettrIE,
    GettrStreamingIE,
)
from .giantbomb import GiantBombIE
from .glide import GlideIE
from .globalplayer import (
    GlobalPlayerAudioEpisodeIE,
    GlobalPlayerAudioIE,
    GlobalPlayerLiveIE,
    GlobalPlayerLivePlaylistIE,
    GlobalPlayerVideoIE,
)
from .globo import (
    GloboArticleIE,
    GloboIE,
)
from .glomex import (
    GlomexEmbedIE,
    GlomexIE,
)
from .gmanetwork import GMANetworkVideoIE
from .go import GoIE
from .godresource import GodResourceIE
from .godtube import GodTubeIE
from .gofile import GofileIE
from .golem import GolemIE
from .goodgame import GoodGameIE
from .googledrive import (
    GoogleDriveFolderIE,
    GoogleDriveIE,
)
from .googlesearch import GoogleSearchIE
from .goplay import GoPlayIE
from .gopro import GoProIE
from .gotostage import GoToStageIE
from .graspop import GraspopIE
from .gronkh import (
    GronkhFeedIE,
    GronkhIE,
    GronkhVodsIE,
)
from .groupon import GrouponIE
from .harpodeon import HarpodeonIE
from .hbo import HBOIE
from .hearthisat import HearThisAtIE
from .heise import HeiseIE
from .hellporno import HellPornoIE
from .hgtv import HGTVComShowIE
from .hidive import HiDiveIE
from .historicfilms import HistoricFilmsIE
from .hitrecord import HitRecordIE
from .hollywoodreporter import (
    HollywoodReporterIE,
    HollywoodReporterPlaylistIE,
)
from .holodex import HolodexIE
from .hotnewhiphop import HotNewHipHopIE
from .hotstar import (
    HotStarIE,
    HotStarPrefixIE,
    HotStarSeriesIE,
)
from .hrefli import HrefLiRedirectIE
from .hrfensehen import HRFernsehenIE
from .hrti import (
    HRTiIE,
    HRTiPlaylistIE,
)
from .hse import (
    HSEProductIE,
    HSEShowIE,
)
from .huajiao import HuajiaoIE
from .huffpost import HuffPostIE
from .hungama import (
    HungamaAlbumPlaylistIE,
    HungamaIE,
    HungamaSongIE,
)
from .huya import (
    HuyaLiveIE,
    HuyaVideoIE,
)
from .hypem import HypemIE
from .hypergryph import MonsterSirenHypergryphMusicIE
from .hytale import HytaleIE
from .icareus import IcareusIE
from .ichinanalive import (
    IchinanaLiveClipIE,
    IchinanaLiveIE,
    IchinanaLiveVODIE,
)
from .idagio import (
    IdagioAlbumIE,
    IdagioPersonalPlaylistIE,
    IdagioPlaylistIE,
    IdagioRecordingIE,
    IdagioTrackIE,
)
from .ign import (
    IGNIE,
    IGNArticleIE,
    IGNVideoIE,
)
from .iheart import (
    IHeartRadioIE,
    IHeartRadioPodcastIE,
)
from .ilpost import IlPostIE
from .iltalehti import IltalehtiIE
from .imdb import (
    ImdbIE,
    ImdbListIE,
)
from .imgur import (
    ImgurAlbumIE,
    ImgurGalleryIE,
    ImgurIE,
)
from .ina import InaIE
from .inc import IncIE
from .indavideo import IndavideoEmbedIE
from .infoq import InfoQIE
from .instagram import (
    InstagramIE,
    InstagramIOSIE,
    InstagramStoryIE,
    InstagramTagIE,
    InstagramUserIE,
)
from .internazionale import InternazionaleIE
from .iprima import (
    IPrimaCNNIE,
    IPrimaIE,
)
from .iqiyi import (
    IqAlbumIE,
    IqIE,
    IqiyiIE,
)
from .islamchannel import (
    IslamChannelIE,
    IslamChannelSeriesIE,
)
from .israelnationalnews import IsraelNationalNewsIE
from .itprotv import (
    ITProTVCourseIE,
    ITProTVIE,
)
from .itv import (
    ITVBTCCIE,
    ITVIE,
)
from .ivi import (
    IviCompilationIE,
    IviIE,
)
from .ivideon import IvideonIE
from .ivoox import IvooxIE
from .iwara import (
    IwaraIE,
    IwaraPlaylistIE,
    IwaraUserIE,
)
from .ixigua import IxiguaIE
from .jamendo import (
    JamendoAlbumIE,
    JamendoIE,
)
from .japandiet import (
    SangiinIE,
    SangiinInstructionIE,
    ShugiinItvLiveIE,
    ShugiinItvLiveRoomIE,
    ShugiinItvVodIE,
)
from .jeuxvideo import JeuxVideoIE
from .jiosaavn import (
    JioSaavnAlbumIE,
    JioSaavnArtistIE,
    JioSaavnPlaylistIE,
    JioSaavnShowIE,
    JioSaavnShowPlaylistIE,
    JioSaavnSongIE,
)
from .joj import JojIE
from .jove import JoveIE
from .jstream import JStreamIE
from .jtbc import (
    JTBCIE,
    JTBCProgramIE,
)
from .jwplatform import JWPlatformIE
from .kakao import KakaoIE
from .kaltura import KalturaIE
from .kankanews import KankaNewsIE
from .karaoketv import KaraoketvIE
from .kelbyone import KelbyOneIE
from .kenh14 import (
    Kenh14PlaylistIE,
    Kenh14VideoIE,
)
from .khanacademy import (
    KhanAcademyIE,
    KhanAcademyUnitIE,
)
from .kick import (
    KickClipIE,
    KickIE,
    KickVODIE,
)
from .kicker import KickerIE
from .kickstarter import KickStarterIE
from .kika import (
    KikaIE,
    KikaPlaylistIE,
)
from .kinopoisk import KinoPoiskIE
from .kommunetv import KommunetvIE
from .kompas import KompasVideoIE
from .krasview import KrasViewIE
from .kth import KTHIE
from .ku6 import Ku6IE
from .kukululive import KukuluLiveIE
from .kuwo import (
    KuwoAlbumIE,
    KuwoCategoryIE,
    KuwoChartIE,
    KuwoIE,
    KuwoMvIE,
    KuwoSingerIE,
)
from .la7 import (
    LA7IE,
    LA7PodcastEpisodeIE,
    LA7PodcastIE,
)
from .laracasts import (
    LaracastsIE,
    LaracastsPlaylistIE,
)
from .lastfm import (
    LastFMIE,
    LastFMPlaylistIE,
    LastFMUserIE,
)
from .laxarxames import LaXarxaMesIE
from .lbry import (
    LBRYIE,
    LBRYChannelIE,
    LBRYPlaylistIE,
)
from .lci import LCIIE
from .lcp import (
    LcpIE,
    LcpPlayIE,
)
from .learningonscreen import LearningOnScreenIE
from .lecture2go import Lecture2GoIE
from .lecturio import (
    LecturioCourseIE,
    LecturioDeCourseIE,
    LecturioIE,
)
from .leeco import (
    LeIE,
    LePlaylistIE,
)
from .lefigaro import (
    LeFigaroVideoEmbedIE,
    LeFigaroVideoSectionIE,
)
from .lego import LEGOIE
from .lemonde import LemondeIE
from .lenta import LentaIE
from .libraryofcongress import LibraryOfCongressIE
from .libsyn import LibsynIE
from .lifenews import (
    LifeEmbedIE,
    LifeNewsIE,
)
from .likee import (
    LikeeIE,
    LikeeUserIE,
)
from .linkedin import (
    LinkedInEventsIE,
    LinkedInIE,
    LinkedInLearningCourseIE,
    LinkedInLearningIE,
)
from .liputan6 import Liputan6IE
from .listennotes import ListenNotesIE
from .litv import LiTVIE
from .livejournal import LiveJournalIE
from .livestreamfails import LivestreamfailsIE
from .lnk import LnkIE
from .locipo import (
    LocipoIE,
    LocipoPlaylistIE,
)
from .loco import LocoIE
from .loom import (
    LoomFolderIE,
    LoomIE,
)
from .lovehomeporn import LoveHomePornIE
from .lrt import (
    LRTVODIE,
    LRTRadioIE,
    LRTStreamIE,
)
from .lsm import (
    LSMLREmbedIE,
    LSMLTVEmbedIE,
    LSMReplayIE,
)
from .lumni import LumniIE
from .maariv import MaarivIE
from .magellantv import MagellanTVIE
from .magentamusik import MagentaMusikIE
from .mailru import (
    MailRuIE,
    MailRuMusicIE,
    MailRuMusicSearchIE,
)
from .mainstreaming import MainStreamingIE
from .mangomolo import (
    MangomoloLiveIE,
    MangomoloVideoIE,
)
from .manyvids import ManyVidsIE
from .maoritv import MaoriTVIE
from .markiza import (
    MarkizaIE,
    MarkizaPageIE,
)
from .massengeschmacktv import MassengeschmackTVIE
from .masters import MastersIE
from .matchitv import MatchiTVIE
from .matchtv import MatchTVIE
from .mave import (
    MaveChannelIE,
    MaveIE,
)
from .mbn import MBNIE
from .mdr import MDRIE
from .medaltv import MedalTVIE
from .mediaite import MediaiteIE
from .mediaklikk import MediaKlikkIE
from .medialaan import MedialaanIE
from .mediaset import (
    MediasetIE,
    MediasetShowIE,
)
from .mediasite import (
    MediasiteCatalogIE,
    MediasiteIE,
    MediasiteNamedCatalogIE,
)
from .mediastream import (
    MediaStreamIE,
    WinSportsVideoIE,
)
from .mediaworksnz import MediaWorksNZVODIE
from .medici import MediciIE
from .megaphone import MegaphoneIE
from .megatvcom import (
    MegaTVComEmbedIE,
    MegaTVComIE,
)
from .meipai import MeipaiIE
from .melonvod import MelonVODIE
from .metacritic import MetacriticIE
from .mgtv import MGTVIE
from .microsoftembed import (
    MicrosoftBuildIE,
    MicrosoftEmbedIE,
    MicrosoftLearnEpisodeIE,
    MicrosoftLearnPlaylistIE,
    MicrosoftLearnSessionIE,
    MicrosoftMediusIE,
)
from .minds import (
    MindsChannelIE,
    MindsGroupIE,
    MindsIE,
)
from .mir24tv import Mir24TvIE
from .mirrativ import (
    MirrativIE,
    MirrativUserIE,
)
from .mirrorcouk import MirrorCoUKIE
from .mit import (
    OCWMITIE,
    TechTVMITIE,
)
from .mixch import (
    MixchArchiveIE,
    MixchIE,
    MixchMovieIE,
)
from .mixcloud import (
    MixcloudIE,
    MixcloudPlaylistIE,
    MixcloudUserIE,
)
from .mixlr import (
    MixlrIE,
    MixlrRecoringIE,
)
from .mlb import (
    MLBIE,
    MLBTVIE,
    MLBArticleIE,
    MLBVideoIE,
)
from .mlssoccer import MLSSoccerIE
from .mocha import MochaVideoIE
from .mojevideo import MojevideoIE
from .monstercat import MonstercatIE
from .motorsport import MotorsportIE
from .moviepilot import MoviepilotIE
from .movingimage import MovingImageIE
from .msn import MSNIE
from .mtv import MTVIE
from .muenchentv import MuenchenTVIE
from .murrtube import (
    MurrtubeIE,
    MurrtubeUserIE,
)
from .museai import MuseAIIE
from .musescore import MuseScoreIE
from .mux import MuxIE
from .mx3 import (
    Mx3IE,
    Mx3NeoIE,
    Mx3VolksmusikIE,
)
from .mxplayer import (
    MxplayerIE,
    MxplayerShowIE,
)
from .myspace import (
    MySpaceAlbumIE,
    MySpaceIE,
)
from .myspass import MySpassIE
from .myvideoge import MyVideoGeIE
from .myvidster import MyVidsterIE
from .mzaalo import MzaaloIE
from .n1 import (
    N1InfoAssetIE,
    N1InfoIIE,
)
from .nascar import NascarClassicsIE
from .nate import (
    NateIE,
    NateProgramIE,
)
from .nationalgeographic import NationalGeographicTVIE
from .naver import (
    NaverIE,
    NaverLiveIE,
)
from .nba import (
    NBAIE,
    NBAChannelIE,
    NBAEmbedIE,
    NBAWatchCollectionIE,
    NBAWatchEmbedIE,
    NBAWatchIE,
)
from .nbc import (
    NBCIE,
    BravoTVIE,
    NBCNewsIE,
    NBCOlympicsIE,
    NBCOlympicsStreamIE,
    NBCSportsIE,
    NBCSportsStreamIE,
    NBCSportsVPlayerIE,
    NBCStationsIE,
    SyfyIE,
)
from .ndr import (
    NDRIE,
    NDREmbedBaseIE,
    NDREmbedIE,
    NJoyEmbedIE,
    NJoyIE,
)
from .ndtv import NDTVIE
from .nebula import (
    NebulaChannelIE,
    NebulaClassIE,
    NebulaIE,
    NebulaSeasonIE,
    NebulaSubscriptionsIE,
)
from .nekohacker import NekoHackerIE
from .nest import (
    NestClipIE,
    NestIE,
)
from .netapp import (
    NetAppCollectionIE,
    NetAppVideoIE,
)
from .neteasemusic import (
    NetEaseMusicAlbumIE,
    NetEaseMusicDjRadioIE,
    NetEaseMusicIE,
    NetEaseMusicListIE,
    NetEaseMusicMvIE,
    NetEaseMusicProgramIE,
    NetEaseMusicSingerIE,
)
from .netzkino import NetzkinoIE
from .newgrounds import (
    NewgroundsIE,
    NewgroundsPlaylistIE,
    NewgroundsUserIE,
)
from .newspicks import NewsPicksIE
from .newsy import NewsyIE
from .nexx import (
    NexxEmbedIE,
    NexxIE,
)
from .nfb import (
    NFBIE,
    NFBSeriesIE,
)
from .nfhsnetwork import NFHSNetworkIE
from .nfl import (
    NFLIE,
    NFLArticleIE,
    NFLPlusEpisodeIE,
    NFLPlusReplayIE,
)
from .nhk import (
    NhkForSchoolBangumiIE,
    NhkForSchoolProgramListIE,
    NhkForSchoolSubjectIE,
    NhkRadioNewsPageIE,
    NhkRadiruIE,
    NhkRadiruLiveIE,
    NhkVodIE,
    NhkVodProgramIE,
)
from .nhl import NHLIE
from .nick import NickIE
from .niconico import (
    NiconicoHistoryIE,
    NiconicoIE,
    NiconicoLiveIE,
    NiconicoPlaylistIE,
    NiconicoSeriesIE,
    NiconicoUserIE,
    NicovideoSearchDateIE,
    NicovideoSearchIE,
    NicovideoSearchURLIE,
    NicovideoTagURLIE,
)
from .niconicochannelplus import (
    NiconicoChannelPlusChannelLivesIE,
    NiconicoChannelPlusChannelVideosIE,
    NiconicoChannelPlusIE,
)
from .ninaprotocol import NinaProtocolIE
from .ninecninemedia import (
    CPTwentyFourIE,
    NineCNineMediaIE,
)
from .ninegag import NineGagIE
from .ninenews import NineNewsIE
from .ninenow import NineNowIE
from .nintendo import NintendoIE
from .nitter import NitterIE
from .nobelprize import NobelPrizeIE
from .noice import NoicePodcastIE
from .nonktube import NonkTubeIE
from .noodlemagazine import NoodleMagazineIE
from .nosnl import NOSNLArticleIE
from .nova import (
    NovaEmbedIE,
    NovaIE,
)
from .novaplay import NovaPlayIE
from .nowcanal import NowCanalIE
from .nowness import (
    NownessIE,
    NownessPlaylistIE,
    NownessSeriesIE,
)
from .noz import NozIE
from .npo import (
    NPOIE,
    VPROIE,
    WNLIE,
    AndereTijdenIE,
    HetKlokhuisIE,
    NPOLiveIE,
    NPORadioFragmentIE,
    NPORadioIE,
    SchoolTVIE,
)
from .npr import NprIE
from .nrk import (
    NRKIE,
    NRKTVIE,
    NRKPlaylistIE,
    NRKRadioPodkastIE,
    NRKSkoleIE,
    NRKTVDirekteIE,
    NRKTVEpisodeIE,
    NRKTVEpisodesIE,
    NRKTVSeasonIE,
    NRKTVSeriesIE,
)
from .nrl import NRLTVIE
from .nts import NTSLiveIE
from .ntvcojp import NTVCoJpCUIE
from .ntvde import NTVDeIE
from .ntvru import NTVRuIE
from .nubilesporn import NubilesPornIE
from .nuvid import NuvidIE
from .nytimes import (
    NYTimesArticleIE,
    NYTimesCookingIE,
    NYTimesCookingRecipeIE,
    NYTimesIE,
)
from .nzherald import NZHeraldIE
from .nzonscreen import NZOnScreenIE
from .nzz import NZZIE
from .odkmedia import OnDemandChinaEpisodeIE
from .odnoklassniki import OdnoklassnikiIE
from .oftv import (
    OfTVIE,
    OfTVPlaylistIE,
)
from .oktoberfesttv import OktoberfestTVIE
from .olympics import OlympicsReplayIE
from .on24 import On24IE
from .ondemandkorea import (
    OnDemandKoreaIE,
    OnDemandKoreaProgramIE,
)
from .onefootball import OneFootballIE
from .onenewsnz import OneNewsNZIE
from .oneplace import OnePlacePodcastIE
from .onet import (
    OnetChannelIE,
    OnetIE,
    OnetMVPIE,
    OnetPlIE,
)
from .onsen import OnsenIE
from .opencast import (
    OpencastIE,
    OpencastPlaylistIE,
)
from .openrec import (
    OpenRecCaptureIE,
    OpenRecIE,
    OpenRecMovieIE,
)
from .orf import (
    ORFIPTVIE,
    ORFONIE,
    ORFFM4StoryIE,
    ORFPodcastIE,
    ORFRadioIE,
)
from .outsidetv import OutsideTVIE
from .owncloud import OwnCloudIE
from .packtpub import (
    PacktPubCourseIE,
    PacktPubIE,
)
from .palcomp3 import (
    PalcoMP3ArtistIE,
    PalcoMP3IE,
    PalcoMP3VideoIE,
)
from .pandatv import PandaTvIE
from .panopto import (
    PanoptoIE,
    PanoptoListIE,
    PanoptoPlaylistIE,
)
from .parler import ParlerIE
from .parlview import ParlviewIE
from .parti import (
    PartiLivestreamIE,
    PartiVideoIE,
)
from .patreon import (
    PatreonCampaignIE,
    PatreonIE,
)
from .pbs import (
    PBSIE,
    PBSKidsIE,
)
from .pearvideo import PearVideoIE
from .peekvids import (
    PeekVidsIE,
    PlayVidsIE,
)
from .peertube import (
    PeerTubeIE,
    PeerTubePlaylistIE,
)
from .peertv import PeerTVIE
from .peloton import (
    PelotonIE,
    PelotonLiveIE,
)
from .performgroup import PerformGroupIE
from .periscope import (
    PeriscopeIE,
    PeriscopeUserIE,
)
from .pgatour import PGATourIE
from .philharmoniedeparis import PhilharmonieDeParisIE
from .phoenix import PhoenixIE
from .photobucket import PhotobucketIE
from .pialive import PiaLiveIE
from .piapro import PiaproIE
from .picarto import (
    PicartoIE,
    PicartoVodIE,
)
from .piksel import PikselIE
from .pinkbike import PinkbikeIE
from .pinterest import (
    PinterestCollectionIE,
    PinterestIE,
)
from .platzi import (
    PlatziCourseIE,
    PlatziIE,
)
from .playerfm import PlayerFmIE
from .playsuisse import PlaySuisseIE
from .playtvak import PlaytvakIE
from .pluralsight import (
    PluralsightCourseIE,
    PluralsightIE,
)
from .plutotv import PlutoTVIE
from .plyr import PlyrEmbedIE
from .podbayfm import (
    PodbayFMChannelIE,
    PodbayFMIE,
)
from .podchaser import PodchaserIE
from .podomatic import PodomaticIE
from .pokergo import (
    PokerGoCollectionIE,
    PokerGoIE,
)
from .polsatgo import PolsatGoIE
from .polskieradio import (
    PolskieRadioAuditionIE,
    PolskieRadioCategoryIE,
    PolskieRadioIE,
    PolskieRadioLegacyIE,
    PolskieRadioPlayerIE,
    PolskieRadioPodcastIE,
    PolskieRadioPodcastListIE,
)
from .popcorntimes import PopcorntimesIE
from .popcorntv import PopcornTVIE
from .pornbox import PornboxIE
from .pornflip import PornFlipIE
from .pornhub import (
    PornHubIE,
    PornHubPagedVideoListIE,
    PornHubPlaylistIE,
    PornHubUserIE,
    PornHubUserVideosUploadIE,
)
from .pornotube import PornotubeIE
from .pornovoisines import PornoVoisinesIE
from .pornoxo import PornoXOIE
from .pr0gramm import Pr0grammIE
from .prankcast import (
    PrankCastIE,
    PrankCastPostIE,
)
from .premiershiprugby import PremiershipRugbyIE
from .presstv import PressTVIE
from .projectveritas import ProjectVeritasIE
from .prx import (
    PRXAccountIE,
    PRXSeriesIE,
    PRXSeriesSearchIE,
    PRXStoriesSearchIE,
    PRXStoryIE,
)
from .puhutv import (
    PuhuTVIE,
    PuhuTVSerieIE,
)
from .pyvideo import PyvideoIE
from .qdance import QDanceIE
from .qingting import QingTingIE
from .qqmusic import (
    QQMusicAlbumIE,
    QQMusicIE,
    QQMusicPlaylistIE,
    QQMusicSingerIE,
    QQMusicToplistIE,
    QQMusicVideoIE,
)
from .r7 import (
    R7IE,
    R7ArticleIE,
)
from .radiko import (
    RadikoIE,
    RadikoRadioIE,
)
from .radiocanada import (
    RadioCanadaAudioVideoIE,
    RadioCanadaIE,
)
from .radiode import RadioDeIE
from .radiofrance import (
    FranceCultureIE,
    RadioFranceIE,
    RadioFranceLiveIE,
    RadioFrancePodcastIE,
    RadioFranceProfileIE,
    RadioFranceProgramScheduleIE,
)
from .radiojavan import RadioJavanIE
from .radiokapital import (
    RadioKapitalIE,
    RadioKapitalShowIE,
)
from .radioradicale import RadioRadicaleIE
from .radiozet import RadioZetPodcastIE
from .radlive import (
    RadLiveChannelIE,
    RadLiveIE,
    RadLiveSeasonIE,
)
from .rai import (
    RaiCulturaIE,
    RaiIE,
    RaiNewsIE,
    RaiPlayIE,
    RaiPlayLiveIE,
    RaiPlayPlaylistIE,
    RaiPlaySoundIE,
    RaiPlaySoundLiveIE,
    RaiPlaySoundPlaylistIE,
    RaiSudtirolIE,
)
from .raywenderlich import (
    RayWenderlichCourseIE,
    RayWenderlichIE,
)
from .rbgtum import (
    RbgTumCourseIE,
    RbgTumIE,
    RbgTumNewCourseIE,
)
from .rcs import (
    RCSIE,
    RCSEmbedsIE,
    RCSVariousIE,
)
from .rcti import (
    RCTIPlusIE,
    RCTIPlusSeriesIE,
    RCTIPlusTVIE,
)
from .rds import RDSIE
from .redbee import (
    RTBFIE,
    ParliamentLiveUKIE,
)
from .redbulltv import (
    RedBullEmbedIE,
    RedBullIE,
    RedBullTVIE,
    RedBullTVRrnContentIE,
)
from .reddit import RedditIE
from .redgifs import (
    RedGifsIE,
    RedGifsSearchIE,
    RedGifsUserIE,
)
from .redtube import RedTubeIE
from .rentv import (
    RENTVIE,
    RENTVArticleIE,
)
from .restudy import RestudyIE
from .reuters import ReutersIE
from .reverbnation import ReverbNationIE
from .ridehome import RideHomeIE
from .rinsefm import (
    RinseFMArtistPlaylistIE,
    RinseFMIE,
)
from .rockstargames import RockstarGamesIE
from .rokfin import (
    RokfinChannelIE,
    RokfinIE,
    RokfinSearchIE,
    RokfinStackIE,
)
from .roosterteeth import (
    RoosterTeethIE,
    RoosterTeethSeriesIE,
)
from .rottentomatoes import RottenTomatoesIE
from .roya import RoyaLiveIE
from .rozhlas import (
    MujRozhlasIE,
    RozhlasIE,
    RozhlasVltavaIE,
)
from .rte import (
    RteIE,
    RteRadioIE,
)
from .rtl2 import RTL2IE
from .rtlnl import (
    RTLLuArticleIE,
    RTLLuLiveIE,
    RTLLuRadioIE,
    RTLLuTeleVODIE,
    RtlNlIE,
)
from .rtnews import (
    RTDocumentryIE,
    RTDocumentryPlaylistIE,
    RTNewsIE,
    RuptlyIE,
)
from .rtp import RTPIE
from .rtrfm import RTRFMIE
from .rts import RTSIE
from .rtvcplay import (
    RTVCKalturaIE,
    RTVCPlayEmbedIE,
    RTVCPlayIE,
)
from .rtve import (
    RTVEALaCartaIE,
    RTVEAudioIE,
    RTVELiveIE,
    RTVEProgramIE,
    RTVETelevisionIE,
)
from .rtvs import RTVSIE
from .rtvslo import (
    RTVSLOIE,
    RTVSLOShowIE,
)
from .rudovideo import RudoVideoIE
from .rule34video import Rule34VideoIE
from .rumble import (
    RumbleChannelIE,
    RumbleEmbedIE,
    RumbleIE,
)
from .rutube import (
    RutubeChannelIE,
    RutubeEmbedIE,
    RutubeIE,
    RutubeMovieIE,
    RutubePersonIE,
    RutubePlaylistIE,
    RutubeTagsIE,
)
from .ruutu import RuutuIE
from .ruv import (
    RuvIE,
    RuvSpilaIE,
)
from .s4c import (
    S4CIE,
    S4CSeriesIE,
)
from .safari import (
    SafariApiIE,
    SafariCourseIE,
    SafariIE,
)
from .saitosan import SaitosanIE
from .samplefocus import SampleFocusIE
from .sapo import SapoIE
from .sauceplus import (
    SaucePlusChannelIE,
    SaucePlusIE,
)
from .sbs import SBSIE
from .sbscokr import (
    SBSCoKrAllvodProgramIE,
    SBSCoKrIE,
    SBSCoKrProgramsVodIE,
)
from .screen9 import Screen9IE
from .screencast import ScreencastIE
from .screencastify import ScreencastifyIE
from .screencastomatic import ScreencastOMaticIE
from .screenrec import ScreenRecIE
from .scrippsnetworks import (
    ScrippsNetworksIE,
    ScrippsNetworksWatchIE,
)
from .scrolller import ScrolllerIE
from .sejmpl import SejmIE
from .sen import SenIE
from .senalcolombia import SenalColombiaLiveIE
from .senategov import (
    SenateGovIE,
    SenateISVPIE,
)
from .servus import ServusIE
from .sevenplus import SevenPlusIE
from .sexu import SexuIE
from .seznamzpravy import (
    SeznamZpravyArticleIE,
    SeznamZpravyIE,
)
from .shahid import (
    ShahidIE,
    ShahidShowIE,
)
from .sharepoint import SharePointIE
from .shemaroome import ShemarooMeIE
from .shiey import ShieyIE
from .showroomlive import ShowRoomLiveIE
from .sibnet import SibnetEmbedIE
from .simplecast import (
    SimplecastEpisodeIE,
    SimplecastIE,
    SimplecastPodcastIE,
)
from .sina import SinaIE
from .skeb import SkebIE
from .sky import (
    SkyNewsIE,
    SkyNewsStoryIE,
    SkySportsIE,
    SkySportsNewsIE,
)
from .skyit import (
    CieloTVItIE,
    SkyItArteIE,
    SkyItIE,
    SkyItPlayerIE,
    SkyItVideoIE,
    SkyItVideoLiveIE,
    TV8ItIE,
    TV8ItLiveIE,
    TV8ItPlaylistIE,
)
from .skylinewebcams import SkylineWebcamsIE
from .skynewsarabia import (
    SkyNewsArabiaArticleIE,
    SkyNewsArabiaIE,
)
from .skynewsau import SkyNewsAUIE
from .slideshare import SlideshareIE
from .slideslive import SlidesLiveIE
from .slutload import SlutloadIE
from .smotrim import (
    SmotrimAudioIE,
    SmotrimIE,
    SmotrimLiveIE,
    SmotrimPlaylistIE,
)
from .snapchat import SnapchatSpotlightIE
from .softwhiteunderbelly import SoftWhiteUnderbellyIE
from .sohu import (
    SohuIE,
    SohuVIE,
)
from .sonyliv import (
    SonyLIVIE,
    SonyLIVSeriesIE,
)
from .soundcloud import (
    SoundcloudEmbedIE,
    SoundcloudIE,
    SoundcloudPlaylistIE,
    SoundcloudRelatedIE,
    SoundcloudSearchIE,
    SoundcloudSetIE,
    SoundcloudTrackStationIE,
    SoundcloudUserIE,
    SoundcloudUserPermalinkIE,
)
from .soundgasm import (
    SoundgasmIE,
    SoundgasmProfileIE,
)
from .southpark import (
    SouthParkComBrIE,
    SouthParkCoUkIE,
    SouthParkDeIE,
    SouthParkDkIE,
    SouthParkEsIE,
    SouthParkIE,
    SouthParkLatIE,
)
from .sovietscloset import (
    SovietsClosetIE,
    SovietsClosetPlaylistIE,
)
from .spankbang import (
    SpankBangIE,
    SpankBangPlaylistIE,
)
from .spiegel import SpiegelIE
from .sport5 import Sport5IE
from .sportbox import SportBoxIE
from .sportdeutschland import SportDeutschlandIE
from .spreaker import (
    SpreakerIE,
    SpreakerShowIE,
)
from .sproutvideo import (
    SproutVideoIE,
    VidsIoIE,
)
from .srgssr import (
    SRGSSRIE,
    SRGSSRPlayIE,
)
from .srmediathek import SRMediathekIE
from .stacommu import (
    StacommuLiveIE,
    StacommuVODIE,
    TheaterComplexTownPPVIE,
    TheaterComplexTownVODIE,
)
from .stageplus import StagePlusVODConcertIE
from .startrek import StarTrekIE
from .startv import StarTVIE
from .steam import (
    SteamCommunityBroadcastIE,
    SteamCommunityIE,
    SteamIE,
)
from .storyfire import (
    StoryFireIE,
    StoryFireSeriesIE,
    StoryFireUserIE,
)
from .streaks import StreaksIE
from .streamable import StreamableIE
from .streamcz import StreamCZIE
from .streetvoice import StreetVoiceIE
from .stripchat import StripchatIE
from .stv import STVPlayerIE
from .subsplash import (
    SubsplashIE,
    SubsplashPlaylistIE,
)
from .substack import SubstackIE
from .sunporno import SunPornoIE
from .sverigesradio import (
    SverigesRadioEpisodeIE,
    SverigesRadioPublicationIE,
)
from .svt import (
    SVTPageIE,
    SVTPlayIE,
    SVTSeriesIE,
)
from .sztvhu import SztvHuIE
from .tagesschau import TagesschauIE
from .taptap import (
    TapTapAppIE,
    TapTapAppIntlIE,
    TapTapMomentIE,
    TapTapPostIntlIE,
)
from .tarangplus import (
    TarangPlusEpisodesIE,
    TarangPlusPlaylistIE,
    TarangPlusVideoIE,
)
from .tass import TassIE
from .tbs import TBSIE
from .tbsjp import (
    TBSJPEpisodeIE,
    TBSJPPlaylistIE,
    TBSJPProgramIE,
)
from .teachable import (
    TeachableCourseIE,
    TeachableIE,
)
from .teachertube import (
    TeacherTubeIE,
    TeacherTubeUserIE,
)
from .teachingchannel import TeachingChannelIE
from .teamcoco import (
    ConanClassicIE,
    TeamcocoIE,
)
from .teamtreehouse import TeamTreeHouseIE
from .ted import (
    TedEmbedIE,
    TedPlaylistIE,
    TedSeriesIE,
    TedTalkIE,
)
from .tele5 import Tele5IE
from .tele13 import Tele13IE
from .telebruxelles import TeleBruxellesIE
from .telecaribe import TelecaribePlayIE
from .telecinco import TelecincoIE
from .telegraaf import TelegraafIE
from .telegram import TelegramEmbedIE
from .telemb import TeleMBIE
from .telemundo import TelemundoIE
from .telequebec import (
    TeleQuebecEmissionIE,
    TeleQuebecIE,
    TeleQuebecLiveIE,
    TeleQuebecSquatIE,
    TeleQuebecVideoIE,
)
from .teletask import TeleTaskIE
from .telewebion import TelewebionIE
from .tencent import (
    IflixEpisodeIE,
    IflixSeriesIE,
    VQQSeriesIE,
    VQQVideoIE,
    WeTvEpisodeIE,
    WeTvSeriesIE,
)
from .tennistv import TennisTVIE
from .tenplay import (
    TenPlayIE,
    TenPlaySeasonIE,
)
from .testurl import TestURLIE
from .tf1 import TF1IE
from .tfo import TFOIE
from .thechosen import (
    TheChosenGroupIE,
    TheChosenIE,
)
from .theguardian import (
    TheGuardianPodcastIE,
    TheGuardianPodcastPlaylistIE,
)
from .thehighwire import TheHighWireIE
from .theintercept import TheInterceptIE
from .theplatform import (
    ThePlatformFeedIE,
    ThePlatformIE,
)
from .thestar import TheStarIE
from .thesun import TheSunIE
from .theweatherchannel import TheWeatherChannelIE
from .thisamericanlife import ThisAmericanLifeIE
from .thisoldhouse import ThisOldHouseIE
from .thisvid import (
    ThisVidIE,
    ThisVidMemberIE,
    ThisVidPlaylistIE,
)
from .threeqsdn import ThreeQSDNIE
from .threespeak import (
    ThreeSpeakIE,
    ThreeSpeakUserIE,
)
from .tiktok import (
    DouyinIE,
    TikTokCollectionIE,
    TikTokEffectIE,
    TikTokIE,
    TikTokLiveIE,
    TikTokSoundIE,
    TikTokTagIE,
    TikTokUserIE,
    TikTokVMIE,
)
from .tmz import TMZIE
from .tnaflix import (
    EMPFlixIE,
    MovieFapIE,
    TNAFlixIE,
    TNAFlixNetworkEmbedIE,
)
from .toggle import (
    MeWatchIE,
    ToggleIE,
)
from .toggo import ToggoIE
from .tonline import TOnlineIE
from .toongoggles import ToonGogglesIE
from .toutiao import ToutiaoIE
from .toutv import TouTvIE
from .toypics import (
    ToypicsIE,
    ToypicsUserIE,
)
from .trovo import (
    TrovoChannelClipIE,
    TrovoChannelVodIE,
    TrovoIE,
    TrovoVodIE,
)
from .trtcocuk import TrtCocukVideoIE
from .trtworld import TrtWorldIE
from .trueid import TrueIDIE
from .trunews import TruNewsIE
from .truth import TruthIE
from .tube8 import Tube8IE
from .tubetugraz import (
    TubeTuGrazIE,
    TubeTuGrazSeriesIE,
)
from .tubitv import (
    TubiTvIE,
    TubiTvShowIE,
)
from .tumblr import TumblrIE
from .tunein import (
    TuneInEmbedIE,
    TuneInPodcastEpisodeIE,
    TuneInPodcastIE,
    TuneInShortenerIE,
    TuneInStationIE,
)
from .tv2 import (
    TV2IE,
    KatsomoIE,
    MTVUutisetArticleIE,
    TV2ArticleIE,
)
from .tv2dk import (
    TV2DKIE,
    TV2DKBornholmPlayIE,
)
from .tv2hu import (
    TV2HuIE,
    TV2HuSeriesIE,
)
from .tv4 import TV4IE
from .tv5mondeplus import TV5MondePlusIE
from .tv5unis import (
    TV5UnisIE,
    TV5UnisVideoIE,
)
from .tv24ua import TV24UAVideoIE
from .tva import TVAIE
from .tvanouvelles import (
    TVANouvellesArticleIE,
    TVANouvellesIE,
)
from .tvc import (
    TVCIE,
    TVCArticleIE,
)
from .tver import (
    TVerIE,
    TVerOlympicIE,
)
from .tvigle import TvigleIE
from .tviplayer import TVIPlayerIE
from .tvn24 import TVN24IE
from .tvnoe import TVNoeIE
from .tvo import TvoIE
from .tvopengr import (
    TVOpenGrEmbedIE,
    TVOpenGrWatchIE,
)
from .tvp import (
    TVPIE,
    TVPEmbedIE,
    TVPStreamIE,
    TVPVODSeriesIE,
    TVPVODVideoIE,
)
from .tvplay import (
    TVPlayHomeIE,
    TVPlayIE,
)
from .tvw import (
    TvwIE,
    TvwNewsIE,
    TvwTvChannelsIE,
)
from .tweakers import TweakersIE
from .twentymin import TwentyMinutenIE
from .twentythreevideo import TwentyThreeVideoIE
from .twitcasting import (
    TwitCastingIE,
    TwitCastingLiveIE,
    TwitCastingUserIE,
)
from .twitch import (
    TwitchClipsIE,
    TwitchCollectionIE,
    TwitchStreamIE,
    TwitchVideosClipsIE,
    TwitchVideosCollectionsIE,
    TwitchVideosIE,
    TwitchVodIE,
)
from .twitter import (
    TwitterAmplifyIE,
    TwitterBroadcastIE,
    TwitterCardIE,
    TwitterIE,
    TwitterShortenerIE,
    TwitterSpacesIE,
)
from .txxx import (
    PornTopIE,
    TxxxIE,
)
from .udemy import (
    UdemyCourseIE,
    UdemyIE,
)
from .udn import UDNEmbedIE
from .ufctv import UFCTVIE
from .ukcolumn import UkColumnIE
from .uliza import (
    UlizaPlayerIE,
    UlizaPortalIE,
)
from .umg import UMGDeIE
from .unistra import UnistraIE
from .unitednations import UnitedNationsWebTvIE
from .unity import UnityIE
from .unsupported import (
    KnownDRMIE,
    KnownPiracyIE,
)
from .uol import UOLIE
from .uplynk import (
    UplynkIE,
    UplynkPreplayIE,
)
from .urort import UrortIE
from .urplay import URPlayIE
from .usanetwork import USANetworkIE
from .usatoday import USATodayIE
from .ustream import (
    UstreamChannelIE,
    UstreamIE,
)
from .ustudio import (
    UstudioEmbedIE,
    UstudioIE,
)
from .varzesh3 import Varzesh3IE
from .vbox7 import Vbox7IE
from .veo import VeoIE
from .vevo import (
    VevoIE,
    VevoPlaylistIE,
)
from .vgtv import (
    VGTVIE,
    BTArticleIE,
    BTVestlendingenIE,
)
from .vh1 import VH1IE
from .vice import (
    ViceArticleIE,
    ViceIE,
    ViceShowIE,
)
from .viddler import ViddlerIE
from .videa import VideaIE
from .videocampus_sachsen import (
    VideocampusSachsenIE,
    ViMPPlaylistIE,
)
from .videoken import VideoKenPlayerIE
from .videopress import VideoPressIE
from .vidflex import VidflexIE
from .vidio import (
    VidioIE,
    VidioLiveIE,
    VidioPremierIE,
)
from .vidlii import VidLiiIE
from .vidly import VidlyIE
from .vidyard import VidyardIE
from .viewlift import (
    ViewLiftEmbedIE,
    ViewLiftIE,
)
from .viidea import ViideaIE
from .vimeo import (
    VHXEmbedIE,
    VimeoAlbumIE,
    VimeoChannelIE,
    VimeoEventIE,
    VimeoGroupsIE,
    VimeoIE,
    VimeoLikesIE,
    VimeoOndemandIE,
    VimeoProIE,
    VimeoReviewIE,
    VimeoUserIE,
    VimeoWatchLaterIE,
)
from .viously import ViouslyIE
from .viqeo import ViqeoIE
from .visir import VisirIE
from .viu import (
    ViuIE,
    ViuOTTIE,
    ViuOTTIndonesiaIE,
    ViuPlaylistIE,
)
from .vk import (
    VKIE,
    VKPlayIE,
    VKPlayLiveIE,
    VKUserVideosIE,
    VKWallPostIE,
)
from .vocaroo import VocarooIE
from .vodplatform import VODPlatformIE
from .voicy import (
    VoicyChannelIE,
    VoicyIE,
)
from .volejtv import (
    VolejTVCategoryPlaylistIE,
    VolejTVClubPlaylistIE,
    VolejTVIE,
)
from .voxmedia import (
    VoxMediaIE,
    VoxMediaVolumeIE,
)
from .vrsquare import (
    VrSquareChannelIE,
    VrSquareIE,
    VrSquareSearchIE,
    VrSquareSectionIE,
)
from .vrt import (
    VRTIE,
    DagelijkseKostIE,
    Radio1BeIE,
    VrtNUIE,
)
from .vtm import VTMIE
from .vtv import (
    VTVIE,
    VTVGoIE,
)
from .walla import WallaIE
from .washingtonpost import (
    WashingtonPostArticleIE,
    WashingtonPostIE,
)
from .wat import WatIE
from .wdr import (
    WDRIE,
    WDRElefantIE,
    WDRPageIE,
)
from .webcamerapl import WebcameraplIE
from .webcaster import (
    WebcasterFeedIE,
    WebcasterIE,
)
from .webofstories import (
    WebOfStoriesIE,
    WebOfStoriesPlaylistIE,
)
from .weibo import (
    WeiboIE,
    WeiboUserIE,
    WeiboVideoIE,
)
from .weiqitv import WeiqiTVIE
from .weverse import (
    WeverseIE,
    WeverseLiveIE,
    WeverseLiveTabIE,
    WeverseMediaIE,
    WeverseMediaTabIE,
    WeverseMomentIE,
)
from .wevidi import WeVidiIE
from .whowatch import WhoWatchIE
from .whyp import WhypIE
from .wikimedia import WikimediaIE
from .wimbledon import WimbledonIE
from .wimtv import WimTVIE
from .wistia import (
    WistiaChannelIE,
    WistiaIE,
    WistiaPlaylistIE,
)
from .wordpress import (
    WordpressMiniAudioPlayerEmbedIE,
    WordpressPlaylistEmbedIE,
)
from .worldstarhiphop import WorldStarHipHopIE
from .wppilot import (
    WPPilotChannelsIE,
    WPPilotIE,
)
from .wrestleuniverse import (
    WrestleUniversePPVIE,
    WrestleUniverseVODIE,
)
from .wsj import (
    WSJIE,
    WSJArticleIE,
)
from .wwe import WWEIE
from .wykop import (
    WykopDigCommentIE,
    WykopDigIE,
    WykopPostCommentIE,
    WykopPostIE,
)
from .xboxclips import XboxClipsIE
from .xhamster import (
    XHamsterEmbedIE,
    XHamsterIE,
    XHamsterUserIE,
)
from .xiaohongshu import XiaoHongShuIE
from .ximalaya import (
    XimalayaAlbumIE,
    XimalayaIE,
)
from .xinpianchang import XinpianchangIE
from .xminus import XMinusIE
from .xnxx import XNXXIE
from .xvideos import (
    XVideosIE,
    XVideosQuickiesIE,
)
from .xxxymovies import XXXYMoviesIE
from .yahoo import (
    YahooIE,
    YahooJapanNewsIE,
    YahooSearchIE,
)
from .yandexdisk import YandexDiskIE
from .yandexmusic import (
    YandexMusicAlbumIE,
    YandexMusicArtistAlbumsIE,
    YandexMusicArtistTracksIE,
    YandexMusicPlaylistIE,
    YandexMusicTrackIE,
)
from .yandexvideo import (
    YandexVideoIE,
    YandexVideoPreviewIE,
    ZenYandexChannelIE,
    ZenYandexIE,
)
from .yapfiles import YapFilesIE
from .yappy import (
    YappyIE,
    YappyProfileIE,
)
from .yfanefa import YfanefaIE
from .yle_areena import YleAreenaIE
from .youjizz import YouJizzIE
from .youku import (
    YoukuIE,
    YoukuShowIE,
)
from .younow import (
    YouNowChannelIE,
    YouNowLiveIE,
    YouNowMomentIE,
)
from .youporn import (
    YouPornCategoryIE,
    YouPornChannelIE,
    YouPornCollectionIE,
    YouPornIE,
    YouPornStarIE,
    YouPornTagIE,
    YouPornVideosIE,
)
from .youtube import (
    YoutubeClipIE,
    YoutubeConsentRedirectIE,
    YoutubeFavouritesIE,
    YoutubeHistoryIE,
    YoutubeIE,
    YoutubeLivestreamEmbedIE,
    YoutubeMusicSearchURLIE,
    YoutubeNotificationsIE,
    YoutubePlaylistIE,
    YoutubeRecommendedIE,
    YoutubeSearchIE,
    YoutubeSearchURLIE,
    YoutubeShortsAudioPivotIE,
    YoutubeSubscriptionsIE,
    YoutubeTabIE,
    YoutubeTruncatedIDIE,
    YoutubeTruncatedURLIE,
    YoutubeWatchLaterIE,
    YoutubeYtBeIE,
    YoutubeYtUserIE,
)
from .zaiko import (
    ZaikoETicketIE,
    ZaikoIE,
)
from .zapiks import ZapiksIE
from .zattoo import (
    BBVTVIE,
    EWETVIE,
    SAKTVIE,
    VTXTVIE,
    BBVTVLiveIE,
    BBVTVRecordingsIE,
    EinsUndEinsTVIE,
    EinsUndEinsTVLiveIE,
    EinsUndEinsTVRecordingsIE,
    EWETVLiveIE,
    EWETVRecordingsIE,
    GlattvisionTVIE,
    GlattvisionTVLiveIE,
    GlattvisionTVRecordingsIE,
    MNetTVIE,
    MNetTVLiveIE,
    MNetTVRecordingsIE,
    NetPlusTVIE,
    NetPlusTVLiveIE,
    NetPlusTVRecordingsIE,
    OsnatelTVIE,
    OsnatelTVLiveIE,
    OsnatelTVRecordingsIE,
    QuantumTVIE,
    QuantumTVLiveIE,
    QuantumTVRecordingsIE,
    SAKTVLiveIE,
    SAKTVRecordingsIE,
    SaltTVIE,
    SaltTVLiveIE,
    SaltTVRecordingsIE,
    VTXTVLiveIE,
    VTXTVRecordingsIE,
    WalyTVIE,
    WalyTVLiveIE,
    WalyTVRecordingsIE,
    ZattooIE,
    ZattooLiveIE,
    ZattooMoviesIE,
    ZattooRecordingsIE,
)
from .zdf import (
    ZDFIE,
    ZDFChannelIE,
)
from .zeenews import ZeeNewsIE
from .zenporn import ZenPornIE
from .zetland import ZetlandDKArticleIE
from .zhihu import ZhihuIE
from .zingmp3 import (
    ZingMp3AlbumIE,
    ZingMp3ChartHomeIE,
    ZingMp3ChartMusicVideoIE,
    ZingMp3HubIE,
    ZingMp3IE,
    ZingMp3LiveRadioIE,
    ZingMp3PodcastEpisodeIE,
    ZingMp3PodcastIE,
    ZingMp3UserIE,
    ZingMp3WeekChartIE,
)
from .zoom import ZoomIE
from .zype import ZypeIE



import base64
import datetime as dt
import itertools
import json
import re
import time

from .common import InfoExtractor
from ..networking.exceptions import HTTPError
from ..utils import (
    ExtractorError,
    encode_data_uri,
    filter_dict,
    int_or_none,
    jwt_decode_hs256,
    url_or_none,
    urlencode_postdata,
    urljoin,
)
from ..utils.traversal import traverse_obj


class TenPlayIE(InfoExtractor):
    IE_NAME = '10play'
    _VALID_URL = r'https?://(?:www\.)?10(?:play)?\.com\.au/(?:[^/?#]+/)+(?P<id>tpv\d{6}[a-z]{5})'
    _NETRC_MACHINE = '10play'
    _TESTS = [{
        # Geo-restricted to Australia
        'url': 'https://10.com.au/australian-survivor/web-extras/season-10-brains-v-brawn-ii/myless-journey/tpv250414jdmtf',
        'info_dict': {
            'id': '7440980000013868',
            'ext': 'mp4',
            'title': 'Myles\'s Journey',
            'alt_title': 'Myles\'s Journey',
            'description': 'Relive Myles\'s epic Brains V Brawn II journey to reach the game\'s final two',
            'uploader': 'Channel 10',
            'uploader_id': '2199827728001',
            'age_limit': 15,
            'duration': 249,
            'thumbnail': r're:https://.+/.+\.jpg',
            'series': 'Australian Survivor',
            'season': 'Season 10',
            'season_number': 10,
            'timestamp': 1744629420,
            'upload_date': '20250414',
        },
        'params': {'skip_download': 'm3u8'},
    }, {
        # Geo-restricted to Australia
        'url': 'https://10.com.au/neighbours/episodes/season-42/episode-9107/tpv240902nzqyp',
        'info_dict': {
            'id': '9000000000091177',
            'ext': 'mp4',
            'title': 'Neighbours - S42 Ep. 9107',
            'alt_title': 'Thu 05 Sep',
            'description': 'md5:37a1f4271be34b9ee2b533426a5fbaef',
            'duration': 1388,
            'episode': 'Episode 9107',
            'episode_number': 9107,
            'season': 'Season 42',
            'season_number': 42,
            'series': 'Neighbours',
            'thumbnail': r're:https://.+/.+\.jpg',
            'age_limit': 15,
            'timestamp': 1725517860,
            'upload_date': '20240905',
            'uploader': 'Channel 10',
            'uploader_id': '2199827728001',
        },
        'params': {'skip_download': 'm3u8'},
    }, {
        # Geo-restricted to Australia; upgrading the m3u8 quality fails and we need the fallback
        'url': 'https://10.com.au/tiny-chef-show/episodes/season-1/episode-2/tpv240228pofvt',
        'info_dict': {
            'id': '9000000000084116',
            'ext': 'mp4',
            'uploader': 'Channel 10',
            'uploader_id': '2199827728001',
            'duration': 1297,
            'title': 'The Tiny Chef Show - S1 Ep. 2',
            'alt_title': 'S1 Ep. 2 - Popcorn/banana',
            'description': 'md5:d4758b52b5375dfaa67a78261dcb5763',
            'age_limit': 0,
            'series': 'The Tiny Chef Show',
            'season_number': 1,
            'episode_number': 2,
            'timestamp': 1747957740,
            'thumbnail': r're:https://.+/.+\.jpg',
            'upload_date': '20250522',
            'season': 'Season 1',
            'episode': 'Episode 2',
        },
        'params': {'skip_download': 'm3u8'},
        'expected_warnings': ['Failed to download m3u8 information: HTTP Error 502'],
        'skip': 'video unavailable',
    }, {
        'url': 'https://10play.com.au/how-to-stay-married/web-extras/season-1/terrys-talks-ep-1-embracing-change/tpv190915ylupc',
        'only_matching': True,
    }]
    _GEO_BYPASS = False
    _GEO_COUNTRIES = ['AU']
    _AUS_AGES = {
        'G': 0,
        'PG': 15,
        'M': 15,
        'MA': 15,
        'MA15+': 15,
        'R': 18,
        'X': 18,
    }
    _TOKEN_CACHE_KEY = 'token_data'
    _SEGMENT_BITRATE_RE = r'(?m)-(?:300|150|75|55)0000-(\d+(?:-[\da-f]+)?)\.ts$'

    _refresh_token = None
    _access_token = None

    @staticmethod
    def _filter_ads_from_m3u8(m3u8_doc):
        out = []
        for line in m3u8_doc.splitlines():
            if line.startswith('https://redirector.googlevideo.com/'):
                out.pop()
                continue
            out.append(line)

        return '\n'.join(out)

    @staticmethod
    def _generate_xnetwork_ten_auth_token():
        ts = dt.datetime.now(dt.timezone.utc).strftime('%Y%m%d%H%M%S')
        return base64.b64encode(ts.encode()).decode()

    @staticmethod
    def _is_jwt_expired(token):
        return jwt_decode_hs256(token)['exp'] - time.time() < 300

    def _refresh_access_token(self):
        try:
            refresh_data = self._download_json(
                'https://10.com.au/api/token/refresh', None, 'Refreshing access token',
                headers={
                    'Content-Type': 'application/json',
                }, data=json.dumps({
                    'accessToken': self._access_token,
                    'refreshToken': self._refresh_token,
                }).encode())
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError) and e.cause.status == 400:
                self._refresh_token = self._access_token = None
                self.cache.store(self._NETRC_MACHINE, self._TOKEN_CACHE_KEY, [None, None])
                self.report_warning('Refresh token has been invalidated; retrying with credentials')
                self._perform_login(*self._get_login_info())
                return
            raise
        self._access_token = refresh_data['accessToken']
        self._refresh_token = refresh_data['refreshToken']
        self.cache.store(self._NETRC_MACHINE, self._TOKEN_CACHE_KEY, [self._refresh_token, self._access_token])

    def _perform_login(self, username, password):
        if not self._refresh_token:
            self._refresh_token, self._access_token = self.cache.load(
                self._NETRC_MACHINE, self._TOKEN_CACHE_KEY, default=[None, None])
        if self._refresh_token and self._access_token:
            self.write_debug('Using cached refresh token')
            return

        try:
            auth_data = self._download_json(
                'https://10.com.au/api/user/auth', None, 'Logging in',
                headers={
                    'Content-Type': 'application/json',
                    'X-Network-Ten-Auth': self._generate_xnetwork_ten_auth_token(),
                    'Referer': 'https://10.com.au/',
                }, data=json.dumps({
                    'email': username,
                    'password': password,
                }).encode())
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError) and e.cause.status == 400:
                raise ExtractorError('Invalid username/password', expected=True)
            raise

        self._refresh_token = auth_data['jwt']['refreshToken']
        self._access_token = auth_data['jwt']['accessToken']
        self.cache.store(self._NETRC_MACHINE, self._TOKEN_CACHE_KEY, [self._refresh_token, self._access_token])

    def _call_playback_api(self, content_id):
        if self._access_token and self._is_jwt_expired(self._access_token):
            self._refresh_access_token()
        for is_retry in (False, True):
            try:
                return self._download_json_handle(
                    f'https://10.com.au/api/v1/videos/playback/{content_id}/', content_id,
                    note='Downloading video JSON', query={'platform': 'samsung'},
                    headers=filter_dict({
                        'TP-AcceptFeature': 'v1/fw;v1/drm',
                        'Authorization': f'Bearer {self._access_token}' if self._access_token else None,
                    }))
            except ExtractorError as e:
                if not is_retry and isinstance(e.cause, HTTPError) and e.cause.status == 403:
                    if self._access_token:
                        self.to_screen('Access token has expired; refreshing')
                        self._refresh_access_token()
                        continue
                    elif not self._get_login_info()[0]:
                        self.raise_login_required('Login required to access this video', method='password')
                raise

    def _real_extract(self, url):
        content_id = self._match_id(url)
        try:
            data = self._download_json(f'https://10.com.au/api/v1/videos/{content_id}', content_id)
        except ExtractorError as e:
            if (
                isinstance(e.cause, HTTPError) and e.cause.status == 403
                and 'Error 54113' in e.cause.response.read().decode()
            ):
                self.raise_geo_restricted(countries=self._GEO_COUNTRIES)
            raise

        video_data, urlh = self._call_playback_api(content_id)
        content_source_id = video_data['dai']['contentSourceId']
        video_id = video_data['dai']['videoId']
        auth_token = urlh.get_header('x-dai-auth')
        if not auth_token:
            raise ExtractorError('Failed to get DAI auth token')

        dai_data = self._download_json(
            f'https://pubads.g.doubleclick.net/ondemand/hls/content/{content_source_id}/vid/{video_id}/streams',
            content_id, note='Downloading DAI JSON',
            data=urlencode_postdata({'auth-token': auth_token}))

        # Ignore subs to avoid ad break cleanup
        formats, _ = self._extract_m3u8_formats_and_subtitles(
            dai_data['stream_manifest'], content_id, 'mp4')

        already_have_1080p = False
        for fmt in formats:
            m3u8_doc = self._download_webpage(
                fmt['url'], content_id, note='Downloading m3u8 information')
            m3u8_doc = self._filter_ads_from_m3u8(m3u8_doc)
            fmt['hls_media_playlist_data'] = m3u8_doc
            if fmt.get('height') == 1080:
                already_have_1080p = True

        # Attempt format upgrade
        if not already_have_1080p and m3u8_doc and re.search(self._SEGMENT_BITRATE_RE, m3u8_doc):
            m3u8_doc = re.sub(self._SEGMENT_BITRATE_RE, r'-5000000-\1.ts', m3u8_doc)
            m3u8_doc = re.sub(r'-(?:300|150|75|55)0000\.key"', r'-5000000.key"', m3u8_doc)
            formats.append({
                'format_id': 'upgrade-attempt-1080p',
                'url': encode_data_uri(m3u8_doc.encode(), 'application/x-mpegurl'),
                'hls_media_playlist_data': m3u8_doc,
                'width': 1920,
                'height': 1080,
                'ext': 'mp4',
                'protocol': 'm3u8_native',
                '__needs_testing': True,
            })

        return {
            'id': content_id,
            'formats': formats,
            'subtitles': {'en': [{'url': data['captionUrl']}]} if url_or_none(data.get('captionUrl')) else None,
            'uploader': 'Channel 10',
            'uploader_id': '2199827728001',
            **traverse_obj(data, {
                'id': ('altId', {str}),
                'duration': ('duration', {int_or_none}),
                'title': ('subtitle', {str}),
                'alt_title': ('title', {str}),
                'description': ('description', {str}),
                'age_limit': ('classification', {self._AUS_AGES.get}),
                'series': ('tvShow', {str}),
                'season_number': ('season', {int_or_none}),
                'episode_number': ('episode', {int_or_none}),
                'timestamp': ('published', {int_or_none}),
                'thumbnail': ('imageUrl', {url_or_none}),
            }),
        }


class TenPlaySeasonIE(InfoExtractor):
    IE_NAME = '10play:season'
    _VALID_URL = r'https?://(?:www\.)?10(?:play)?\.com\.au/(?P<show>[^/?#]+)/episodes/(?P<season>[^/?#]+)/?(?:$|[?#])'
    _TESTS = [{
        'url': 'https://10.com.au/masterchef/episodes/season-15',
        'info_dict': {
            'title': 'Season 15',
            'id': 'MTQ2NjMxOQ==',
        },
        'playlist_mincount': 50,
    }, {
        'url': 'https://10.com.au/the-bold-and-the-beautiful-fast-tracked/episodes/season-2024',
        'info_dict': {
            'title': 'Season 2024',
            'id': 'Mjc0OTIw',
        },
        'playlist_mincount': 159,
    }, {
        'url': 'https://10play.com.au/the-bold-and-the-beautiful-fast-tracked/episodes/season-2024',
        'only_matching': True,
    }]

    def _entries(self, load_more_url, display_id=None):
        skip_ids = []
        for page in itertools.count(1):
            episodes_carousel = self._download_json(
                load_more_url, display_id, query={'skipIds[]': skip_ids},
                note=f'Fetching episodes page {page}')

            episodes_chunk = episodes_carousel['items']
            skip_ids.extend(ep['id'] for ep in episodes_chunk)

            for ep in episodes_chunk:
                yield ep['cardLink']
            if not episodes_carousel['hasMore']:
                break

    def _real_extract(self, url):
        show, season = self._match_valid_url(url).group('show', 'season')
        season_info = self._download_json(
            f'https://10.com.au/api/shows/{show}/episodes/{season}', f'{show}/{season}')

        episodes_carousel = traverse_obj(season_info, (
            'content', 0, 'components', (
                lambda _, v: v['title'].lower() == 'episodes',
                (..., {dict}),
            )), get_all=False) or {}

        playlist_id = episodes_carousel['tpId']

        return self.playlist_from_matches(
            self._entries(urljoin(url, episodes_carousel['loadMoreUrl']), playlist_id),
            playlist_id, traverse_obj(season_info, ('content', 0, 'title', {str})),
            getter=urljoin(url))



from .common import InfoExtractor
from .rumble import RumbleEmbedIE
from .youtube import YoutubeIE
from ..utils import ExtractorError, clean_html, get_element_by_class, strip_or_none


class Funker530IE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?funker530\.com/video/(?P<id>[^/?#]+)'
    _TESTS = [{
        'url': 'https://funker530.com/video/azov-patrol-caught-in-open-under-automatic-grenade-launcher-fire/',
        'md5': '085f50fea27523a388bbc22e123e09c8',
        'info_dict': {
            'id': 'v2qbmu4',
            'ext': 'mp4',
            'title': 'Azov Patrol Caught In Open Under Automatic Grenade Launcher Fire',
            'thumbnail': r're:^https?://.*\.jpg$',
            'uploader': 'Funker530',
            'channel': 'Funker530',
            'channel_url': 'https://rumble.com/c/c-1199543',
            'width': 1280,
            'height': 720,
            'fps': 25,
            'duration': 27,
            'upload_date': '20230608',
            'timestamp': 1686241321,
            'live_status': 'not_live',
            'description': 'md5:bea2e1f458095414e04b5ac189c2f980',
        },
    }, {
        'url': 'https://funker530.com/video/my-friends-joined-the-russians-civdiv/',
        'md5': 'a42c2933391210662e93e867d7124b70',
        'info_dict': {
            'id': 'k-pk4bOvoac',
            'ext': 'mp4',
            'view_count': int,
            'channel': 'Civ Div',
            'comment_count': int,
            'channel_follower_count': int,
            'thumbnail': 'https://i.ytimg.com/vi/k-pk4bOvoac/maxresdefault.jpg',
            'uploader_id': '@CivDiv',
            'duration': 357,
            'channel_url': 'https://www.youtube.com/channel/UCgsCiwJ88up-YyMHo7hL5-A',
            'tags': [],
            'uploader_url': 'https://www.youtube.com/@CivDiv',
            'channel_id': 'UCgsCiwJ88up-YyMHo7hL5-A',
            'like_count': int,
            'description': 'md5:aef75ec3f59c07a0e39400f609b24429',
            'live_status': 'not_live',
            'age_limit': 0,
            'uploader': 'Civ Div',
            'categories': ['People & Blogs'],
            'title': 'My “Friends” joined the Russians.',
            'availability': 'public',
            'upload_date': '20230608',
            'playable_in_embed': True,
            'heatmap': 'count:100',
        },
    }]

    def _real_extract(self, url):
        display_id = self._match_id(url)
        webpage = self._download_webpage(url, display_id)
        info = {}
        rumble_url = list(RumbleEmbedIE._extract_embed_urls(url, webpage))
        if rumble_url:
            info = {'url': rumble_url[0], 'ie_key': RumbleEmbedIE.ie_key()}
        else:
            youtube_url = list(YoutubeIE._extract_embed_urls(url, webpage))
            if youtube_url:
                info = {'url': youtube_url[0], 'ie_key': YoutubeIE.ie_key()}
        if not info:
            raise ExtractorError('No videos found on webpage', expected=True)

        return {
            **info,
            '_type': 'url_transparent',
            'description': strip_or_none(self._search_regex(
                r'(?s)(.+)About the Author', clean_html(get_element_by_class('video-desc-paragraph', webpage)),
                'description', default=None)),
        }



import json
import re

from .common import InfoExtractor
from ..utils import (
    classproperty,
    int_or_none,
    traverse_obj,
    urljoin,
)


class BrainPOPBaseIE(InfoExtractor):
    _NETRC_MACHINE = 'brainpop'
    _ORIGIN = ''  # So that _VALID_URL doesn't crash
    _LOGIN_ERRORS = {
        1502: 'The username and password you entered did not match.',  # LOGIN_FAILED
        1503: 'Payment method is expired.',  # LOGIN_FAILED_ACCOUNT_NOT_ACTIVE
        1506: 'Your BrainPOP plan has expired.',  # LOGIN_FAILED_ACCOUNT_EXPIRED
        1507: 'Terms not accepted.',  # LOGIN_FAILED_TERMS_NOT_ACCEPTED
        1508: 'Account not activated.',  # LOGIN_FAILED_SUBSCRIPTION_NOT_ACTIVE
        1512: 'The maximum number of devices permitted are logged in with your account right now.',  # LOGIN_FAILED_LOGIN_LIMIT_REACHED
        1513: 'You are trying to access your account from outside of its allowed IP range.',  # LOGIN_FAILED_INVALID_IP
        1514: 'Individual accounts are not included in your plan. Try again with your shared username and password.',  # LOGIN_FAILED_MBP_DISABLED
        1515: 'Account not activated.',  # LOGIN_FAILED_TEACHER_NOT_ACTIVE
        1523: 'That username and password won\'t work on this BrainPOP site.',  # LOGIN_FAILED_NO_ACCESS
        1524: 'You\'ll need to join a class before you can login.',  # LOGIN_FAILED_STUDENT_NO_PERIOD
        1526: 'Your account is locked. Reset your password, or ask a teacher or administrator for help.',  # LOGIN_FAILED_ACCOUNT_LOCKED
    }

    @classproperty
    def _VALID_URL(cls):
        root = re.escape(cls._ORIGIN).replace(r'https:', r'https?:').replace(r'www\.', r'(?:www\.)?')
        return rf'{root}/(?P<slug>[^/]+/[^/]+/(?P<id>[^/?#&]+))'

    def _assemble_formats(self, slug, format_id, display_id, token='', extra_fields={}):
        formats = []
        formats = self._extract_m3u8_formats(
            f'{urljoin(self._HLS_URL, slug)}.m3u8?{token}',
            display_id, 'mp4', m3u8_id=f'{format_id}-hls', fatal=False)
        formats.append({
            'format_id': format_id,
            'url': f'{urljoin(self._VIDEO_URL, slug)}?{token}',
        })
        for f in formats:
            f.update(extra_fields)
        return formats

    def _extract_adaptive_formats(self, data, token, display_id, key_format='%s', extra_fields={}):
        formats = []
        additional_key_formats = {
            '%s': {},
            'ad_%s': {
                'format_note': 'Audio description',
                'source_preference': -2,
            },
        }
        for additional_key_format, additional_key_fields in additional_key_formats.items():
            for key_quality, key_index in enumerate(('high', 'low')):
                full_key_index = additional_key_format % (key_format % key_index)
                if data.get(full_key_index):
                    formats.extend(self._assemble_formats(data[full_key_index], full_key_index, display_id, token, {
                        'quality': -1 - key_quality,
                        **additional_key_fields,
                        **extra_fields,
                    }))
        return formats

    def _perform_login(self, username, password):
        login_res = self._download_json(
            'https://api.brainpop.com/api/login', None,
            data=json.dumps({'username': username, 'password': password}).encode(),
            headers={
                'Content-Type': 'application/json',
                'Referer': self._ORIGIN,
            }, note='Logging in', errnote='Unable to log in', expected_status=400)
        status_code = int_or_none(login_res['status_code'])
        if status_code != 1505:
            self.report_warning(
                f'Unable to login: {self._LOGIN_ERRORS.get(status_code) or login_res.get("message")}'
                or f'Got status code {status_code}')


class BrainPOPIE(BrainPOPBaseIE):
    _ORIGIN = 'https://www.brainpop.com'
    _VIDEO_URL = 'https://svideos.brainpop.com'
    _HLS_URL = 'https://hls.brainpop.com'
    _CDN_URL = 'https://cdn.brainpop.com'
    _TESTS = [{
        'url': 'https://www.brainpop.com/health/conflictresolution/martinlutherkingjr/movie?ref=null',
        'md5': '3ead374233ae74c7f1b0029a01c972f0',
        'info_dict': {
            'id': '1f3259fa457292b4',
            'ext': 'mp4',
            'title': 'Martin Luther King, Jr.',
            'display_id': 'martinlutherkingjr',
            'description': 'md5:f403dbb2bf3ccc7cf4c59d9e43e3c349',
        },
    }, {
        'url': 'https://www.brainpop.com/science/space/bigbang/',
        'md5': '9a1ff0e77444dd9e437354eb669c87ec',
        'info_dict': {
            'id': 'acae52cd48c99acf',
            'ext': 'mp4',
            'title': 'Big Bang',
            'display_id': 'bigbang',
            'description': 'md5:3e53b766b0f116f631b13f4cae185d38',
        },
        'skip': 'Requires login',
    }]

    def _real_extract(self, url):
        slug, display_id = self._match_valid_url(url).group('slug', 'id')
        movie_data = self._download_json(
            f'https://api.brainpop.com/api/content/published/bp/en/{slug}/movie?full=1', display_id,
            'Downloading movie data JSON', 'Unable to download movie data')['data']
        topic_data = traverse_obj(self._download_json(
            f'https://api.brainpop.com/api/content/published/bp/en/{slug}?full=1', display_id,
            'Downloading topic data JSON', 'Unable to download topic data', fatal=False),
            ('data', 'topic'), expected_type=dict) or movie_data['topic']

        if not traverse_obj(movie_data, ('access', 'allow')):
            reason = traverse_obj(movie_data, ('access', 'reason'))
            if 'logged' in reason:
                self.raise_login_required(reason, metadata_available=True)
            else:
                self.raise_no_formats(reason, video_id=display_id)
        movie_feature = movie_data['feature']
        movie_feature_data = movie_feature['data']

        formats, subtitles = [], {}
        formats.extend(self._extract_adaptive_formats(movie_feature_data, movie_feature_data.get('token', ''), display_id, '%s_v2', {
            'language': movie_feature.get('language') or 'en',
            'language_preference': 10,
        }))
        for lang, localized_feature in traverse_obj(movie_feature, 'localization', default={}, expected_type=dict).items():
            formats.extend(self._extract_adaptive_formats(localized_feature, localized_feature.get('token', ''), display_id, '%s_v2', {
                'language': lang,
                'language_preference': -10,
            }))

        # TODO: Do localization fields also have subtitles?
        for name, url in movie_feature_data.items():
            lang = self._search_regex(
                r'^subtitles_(?P<lang>\w+)$', name, 'subtitle metadata', default=None)
            if lang and url:
                subtitles.setdefault(lang, []).append({
                    'url': urljoin(self._CDN_URL, url),
                })

        return {
            'id': topic_data['topic_id'],
            'display_id': display_id,
            'title': topic_data.get('name'),
            'description': topic_data.get('synopsis'),
            'formats': formats,
            'subtitles': subtitles,
        }


class BrainPOPLegacyBaseIE(BrainPOPBaseIE):
    def _parse_js_topic_data(self, topic_data, display_id, token):
        movie_data = topic_data['movies']
        # TODO: Are there non-burned subtitles?
        formats = self._extract_adaptive_formats(movie_data, token, display_id)

        return {
            'id': topic_data['EntryID'],
            'display_id': display_id,
            'title': topic_data.get('name'),
            'alt_title': topic_data.get('title'),
            'description': topic_data.get('synopsis'),
            'formats': formats,
        }

    def _real_extract(self, url):
        display_id = self._match_id(url)
        webpage = self._download_webpage(url, display_id)
        topic_data = self._search_json(
            r'var\s+content\s*=\s*', webpage, 'content data',
            display_id, end_pattern=';')['category']['unit']['topic']
        token = self._search_regex(r'ec_token\s*:\s*[\'"]([^\'"]+)', webpage, 'video token')
        return self._parse_js_topic_data(topic_data, display_id, token)


class BrainPOPJrIE(BrainPOPLegacyBaseIE):
    _ORIGIN = 'https://jr.brainpop.com'
    _VIDEO_URL = 'https://svideos-jr.brainpop.com'
    _HLS_URL = 'https://hls-jr.brainpop.com'
    _CDN_URL = 'https://cdn-jr.brainpop.com'
    _TESTS = [{
        'url': 'https://jr.brainpop.com/health/feelingsandsel/emotions/',
        'md5': '04e0561bb21770f305a0ce6cf0d869ab',
        'info_dict': {
            'id': '347',
            'ext': 'mp4',
            'title': 'Emotions',
            'display_id': 'emotions',
        },
    }, {
        'url': 'https://jr.brainpop.com/science/habitats/arctichabitats/',
        'md5': 'b0ed063bbd1910df00220ee29340f5d6',
        'info_dict': {
            'id': '29',
            'ext': 'mp4',
            'title': 'Arctic Habitats',
            'display_id': 'arctichabitats',
        },
        'skip': 'Requires login',
    }]


class BrainPOPELLIE(BrainPOPLegacyBaseIE):
    _ORIGIN = 'https://ell.brainpop.com'
    _VIDEO_URL = 'https://svideos-esl.brainpop.com'
    _HLS_URL = 'https://hls-esl.brainpop.com'
    _CDN_URL = 'https://cdn-esl.brainpop.com'
    _TESTS = [{
        'url': 'https://ell.brainpop.com/level1/unit1/lesson1/',
        'md5': 'a2012700cfb774acb7ad2e8834eed0d0',
        'info_dict': {
            'id': '1',
            'ext': 'mp4',
            'title': 'Lesson 1',
            'display_id': 'lesson1',
            'alt_title': 'Personal Pronouns',
        },
    }, {
        'url': 'https://ell.brainpop.com/level3/unit6/lesson5/',
        'md5': 'be19c8292c87b24aacfb5fda2f3f8363',
        'info_dict': {
            'id': '101',
            'ext': 'mp4',
            'title': 'Lesson 5',
            'display_id': 'lesson5',
            'alt_title': 'Review: Unit 6',
        },
        'skip': 'Requires login',
    }]


class BrainPOPEspIE(BrainPOPLegacyBaseIE):
    IE_DESC = 'BrainPOP Español'
    _ORIGIN = 'https://esp.brainpop.com'
    _VIDEO_URL = 'https://svideos.brainpop.com'
    _HLS_URL = 'https://hls.brainpop.com'
    _CDN_URL = 'https://cdn.brainpop.com/mx'
    _TESTS = [{
        'url': 'https://esp.brainpop.com/ciencia/la_diversidad_de_la_vida/ecosistemas/',
        'md5': 'cb3f062db2b3c5240ddfcfde7108f8c9',
        'info_dict': {
            'id': '3893',
            'ext': 'mp4',
            'title': 'Ecosistemas',
            'display_id': 'ecosistemas',
            'description': 'md5:80fc55b07e241f8c8f2aa8d74deaf3c3',
        },
    }, {
        'url': 'https://esp.brainpop.com/espanol/la_escritura/emily_dickinson/',
        'md5': '98c1b9559e0e33777209c425cda7dac4',
        'info_dict': {
            'id': '7146',
            'ext': 'mp4',
            'title': 'Emily Dickinson',
            'display_id': 'emily_dickinson',
            'description': 'md5:2795ad87b1d239c9711c1e92ab5a978b',
        },
        'skip': 'Requires login',
    }]


class BrainPOPFrIE(BrainPOPLegacyBaseIE):
    IE_DESC = 'BrainPOP Français'
    _ORIGIN = 'https://fr.brainpop.com'
    _VIDEO_URL = 'https://svideos.brainpop.com'
    _HLS_URL = 'https://hls.brainpop.com'
    _CDN_URL = 'https://cdn.brainpop.com/fr'
    _TESTS = [{
        'url': 'https://fr.brainpop.com/sciencesdelaterre/energie/sourcesdenergie/',
        'md5': '97e7f48af8af93f8a2be11709f239371',
        'info_dict': {
            'id': '1651',
            'ext': 'mp4',
            'title': 'Sources d\'énergie',
            'display_id': 'sourcesdenergie',
            'description': 'md5:7eece350f019a21ef9f64d4088b2d857',
        },
    }, {
        'url': 'https://fr.brainpop.com/francais/ecrire/plagiat/',
        'md5': '0cf2b4f89804d0dd4a360a51310d445a',
        'info_dict': {
            'id': '5803',
            'ext': 'mp4',
            'title': 'Plagiat',
            'display_id': 'plagiat',
            'description': 'md5:4496d87127ace28e8b1eda116e77cd2b',
        },
        'skip': 'Requires login',
    }]


class BrainPOPIlIE(BrainPOPLegacyBaseIE):
    IE_DESC = 'BrainPOP Hebrew'
    _ORIGIN = 'https://il.brainpop.com'
    _VIDEO_URL = 'https://svideos.brainpop.com'
    _HLS_URL = 'https://hls.brainpop.com'
    _CDN_URL = 'https://cdn.brainpop.com/he'
    _TESTS = [{
        'url': 'https://il.brainpop.com/category_9/subcategory_150/subjects_3782/',
        'md5': '9e4ea9dc60ecd385a6e5ca12ccf31641',
        'info_dict': {
            'id': '3782',
            'ext': 'mp4',
            'title': 'md5:e993632fcda0545d9205602ec314ad67',
            'display_id': 'subjects_3782',
            'description': 'md5:4cc084a8012beb01f037724423a4d4ed',
        },
    }]



import re
import urllib.parse

from .common import InfoExtractor
from ..utils import (
    ExtractorError,
    InAdvancePagedList,
    clean_html,
    get_element_by_id,
    remove_start,
)


class KuwoBaseIE(InfoExtractor):
    _FORMATS = [
        {'format': 'ape', 'ext': 'ape', 'preference': 100},
        {'format': 'mp3-320', 'ext': 'mp3', 'br': '320kmp3', 'abr': 320, 'preference': 80},
        {'format': 'mp3-192', 'ext': 'mp3', 'br': '192kmp3', 'abr': 192, 'preference': 70},
        {'format': 'mp3-128', 'ext': 'mp3', 'br': '128kmp3', 'abr': 128, 'preference': 60},
        {'format': 'wma', 'ext': 'wma', 'preference': 20},
        {'format': 'aac', 'ext': 'aac', 'abr': 48, 'preference': 10},
    ]

    def _get_formats(self, song_id, tolerate_ip_deny=False):
        formats = []
        for file_format in self._FORMATS:
            query = {
                'format': file_format['ext'],
                'br': file_format.get('br', ''),
                'rid': f'MUSIC_{song_id}',
                'type': 'convert_url',
                'response': 'url',
            }

            song_url = self._download_webpage(
                'http://antiserver.kuwo.cn/anti.s',
                song_id, note='Download {} url info'.format(file_format['format']),
                query=query, headers=self.geo_verification_headers(),
            )

            if song_url == 'IPDeny' and not tolerate_ip_deny:
                raise ExtractorError('This song is blocked in this region', expected=True)

            if song_url.startswith(('http://', 'https://')):
                formats.append({
                    'url': song_url,
                    'format_id': file_format['format'],
                    'format': file_format['format'],
                    'quality': file_format['preference'],
                    'abr': file_format.get('abr'),
                })

        return formats


class KuwoIE(KuwoBaseIE):
    _WORKING = False
    IE_NAME = 'kuwo:song'
    IE_DESC = '酷我音乐'
    _VALID_URL = r'https?://(?:www\.)?kuwo\.cn/yinyue/(?P<id>\d+)'
    _TESTS = [{
        'url': 'http://www.kuwo.cn/yinyue/635632/',
        'info_dict': {
            'id': '635632',
            'ext': 'ape',
            'title': '爱我别走',
            'creator': '张震岳',
            'upload_date': '20080122',
            'description': 'md5:ed13f58e3c3bf3f7fd9fbc4e5a7aa75c',
        },
        'skip': 'this song has been offline because of copyright issues',
    }, {
        'url': 'http://www.kuwo.cn/yinyue/6446136/',
        'info_dict': {
            'id': '6446136',
            'ext': 'mp3',
            'title': '心',
            'description': 'md5:5d0e947b242c35dc0eb1d2fce9fbf02c',
            'creator': 'IU',
            'upload_date': '20150518',
        },
        'params': {
            'format': 'mp3-320',
        },
    }, {
        'url': 'http://www.kuwo.cn/yinyue/3197154?catalog=yueku2016',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        song_id = self._match_id(url)
        webpage, urlh = self._download_webpage_handle(
            url, song_id, note='Download song detail info',
            errnote='Unable to get song detail info')
        if song_id not in urlh.url or '对不起，该歌曲由于版权问题已被下线，将返回网站首页' in webpage:
            raise ExtractorError('this song has been offline because of copyright issues', expected=True)

        song_name = self._html_search_regex(
            r'<p[^>]+id="lrcName">([^<]+)</p>', webpage, 'song name')
        singer_name = remove_start(self._html_search_regex(
            r'<a[^>]+href="http://www\.kuwo\.cn/artist/content\?name=([^"]+)">',
            webpage, 'singer name', fatal=False), '歌手')
        lrc_content = clean_html(get_element_by_id('lrcContent', webpage))
        if lrc_content == '暂无':     # indicates no lyrics
            lrc_content = None

        formats = self._get_formats(song_id)

        album_id = self._html_search_regex(
            r'<a[^>]+href="http://www\.kuwo\.cn/album/(\d+)/"',
            webpage, 'album id', fatal=False)

        publish_time = None
        if album_id is not None:
            album_info_page = self._download_webpage(
                f'http://www.kuwo.cn/album/{album_id}/', song_id,
                note='Download album detail info',
                errnote='Unable to get album detail info')

            publish_time = self._html_search_regex(
                r'发行时间：(\d{4}-\d{2}-\d{2})', album_info_page,
                'publish time', fatal=False)
            if publish_time:
                publish_time = publish_time.replace('-', '')

        return {
            'id': song_id,
            'title': song_name,
            'creator': singer_name,
            'upload_date': publish_time,
            'description': lrc_content,
            'formats': formats,
        }


class KuwoAlbumIE(InfoExtractor):
    _WORKING = False
    IE_NAME = 'kuwo:album'
    IE_DESC = '酷我音乐 - 专辑'
    _VALID_URL = r'https?://(?:www\.)?kuwo\.cn/album/(?P<id>\d+?)/'
    _TEST = {
        'url': 'http://www.kuwo.cn/album/502294/',
        'info_dict': {
            'id': '502294',
            'title': 'Made\xa0Series\xa0《M》',
            'description': 'md5:d463f0d8a0ff3c3ea3d6ed7452a9483f',
        },
        'playlist_count': 2,
    }

    def _real_extract(self, url):
        album_id = self._match_id(url)

        webpage = self._download_webpage(
            url, album_id, note='Download album info',
            errnote='Unable to get album info')

        album_name = self._html_search_regex(
            r'<div[^>]+class="comm"[^<]+<h1[^>]+title="([^"]+)"', webpage,
            'album name')
        album_intro = remove_start(
            clean_html(get_element_by_id('intro', webpage)),
            f'{album_name}简介：')

        entries = [
            self.url_result(song_url, 'Kuwo') for song_url in re.findall(
                r'<p[^>]+class="listen"><a[^>]+href="(http://www\.kuwo\.cn/yinyue/\d+/)"',
                webpage)
        ]
        return self.playlist_result(entries, album_id, album_name, album_intro)


class KuwoChartIE(InfoExtractor):
    _WORKING = False
    IE_NAME = 'kuwo:chart'
    IE_DESC = '酷我音乐 - 排行榜'
    _VALID_URL = r'https?://yinyue\.kuwo\.cn/billboard_(?P<id>[^.]+).htm'
    _TEST = {
        'url': 'http://yinyue.kuwo.cn/billboard_香港中文龙虎榜.htm',
        'info_dict': {
            'id': '香港中文龙虎榜',
        },
        'playlist_mincount': 7,
    }

    def _real_extract(self, url):
        chart_id = self._match_id(url)
        webpage = self._download_webpage(
            url, chart_id, note='Download chart info',
            errnote='Unable to get chart info')

        entries = [
            self.url_result(song_url, 'Kuwo') for song_url in re.findall(
                r'<a[^>]+href="(http://www\.kuwo\.cn/yinyue/\d+)', webpage)
        ]
        return self.playlist_result(entries, chart_id)


class KuwoSingerIE(InfoExtractor):
    _WORKING = False
    IE_NAME = 'kuwo:singer'
    IE_DESC = '酷我音乐 - 歌手'
    _VALID_URL = r'https?://(?:www\.)?kuwo\.cn/mingxing/(?P<id>[^/]+)'
    _TESTS = [{
        'url': 'http://www.kuwo.cn/mingxing/bruno+mars/',
        'info_dict': {
            'id': 'bruno+mars',
            'title': 'Bruno\xa0Mars',
        },
        'playlist_mincount': 329,
    }, {
        'url': 'http://www.kuwo.cn/mingxing/Ali/music.htm',
        'info_dict': {
            'id': 'Ali',
            'title': 'Ali',
        },
        'playlist_mincount': 95,
        'skip': 'Regularly stalls travis build',  # See https://travis-ci.org/ytdl-org/youtube-dl/jobs/78878540
    }]

    PAGE_SIZE = 15

    def _real_extract(self, url):
        singer_id = self._match_id(url)
        webpage = self._download_webpage(
            url, singer_id, note='Download singer info',
            errnote='Unable to get singer info')

        singer_name = self._html_search_regex(
            r'<h1>([^<]+)</h1>', webpage, 'singer name')

        artist_id = self._html_search_regex(
            r'data-artistid="(\d+)"', webpage, 'artist id')

        page_count = int(self._html_search_regex(
            r'data-page="(\d+)"', webpage, 'page count'))

        def page_func(page_num):
            webpage = self._download_webpage(
                'http://www.kuwo.cn/artist/contentMusicsAjax',
                singer_id, note=f'Download song list page #{page_num + 1}',
                errnote=f'Unable to get song list page #{page_num + 1}',
                query={'artistId': artist_id, 'pn': page_num, 'rn': self.PAGE_SIZE})

            return [
                self.url_result(urllib.parse.urljoin(url, song_url), 'Kuwo')
                for song_url in re.findall(
                    r'<div[^>]+class="name"><a[^>]+href="(/yinyue/\d+)',
                    webpage)
            ]

        entries = InAdvancePagedList(page_func, page_count, self.PAGE_SIZE)

        return self.playlist_result(entries, singer_id, singer_name)


class KuwoCategoryIE(InfoExtractor):
    _WORKING = False
    IE_NAME = 'kuwo:category'
    IE_DESC = '酷我音乐 - 分类'
    _VALID_URL = r'https?://yinyue\.kuwo\.cn/yy/cinfo_(?P<id>\d+?).htm'
    _TEST = {
        'url': 'http://yinyue.kuwo.cn/yy/cinfo_86375.htm',
        'info_dict': {
            'id': '86375',
            'title': '八十年代精选',
            'description': '这些都是属于八十年代的回忆！',
        },
        'playlist_mincount': 24,
    }

    def _real_extract(self, url):
        category_id = self._match_id(url)
        webpage = self._download_webpage(
            url, category_id, note='Download category info',
            errnote='Unable to get category info')

        category_name = self._html_search_regex(
            r'<h1[^>]+title="([^<>]+?)">[^<>]+?</h1>', webpage, 'category name')

        category_desc = remove_start(
            get_element_by_id('intro', webpage).strip(),
            f'{category_name}简介：')
        if category_desc == '暂无':
            category_desc = None

        jsonm = self._parse_json(self._html_search_regex(
            r'var\s+jsonm\s*=\s*([^;]+);', webpage, 'category songs'), category_id)

        entries = [
            self.url_result('http://www.kuwo.cn/yinyue/{}/'.format(song['musicrid']), 'Kuwo')
            for song in jsonm['musiclist']
        ]
        return self.playlist_result(entries, category_id, category_name, category_desc)


class KuwoMvIE(KuwoBaseIE):
    _WORKING = False
    IE_NAME = 'kuwo:mv'
    IE_DESC = '酷我音乐 - MV'
    _VALID_URL = r'https?://(?:www\.)?kuwo\.cn/mv/(?P<id>\d+?)/'
    _TEST = {
        'url': 'http://www.kuwo.cn/mv/6480076/',
        'info_dict': {
            'id': '6480076',
            'ext': 'mp4',
            'title': 'My HouseMV',
            'creator': '2PM',
        },
        # In this video, music URLs (anti.s) are blocked outside China and
        # USA, while the MV URL (mvurl) is available globally, so force the MV
        # URL for consistent results in different countries
        'params': {
            'format': 'mv',
        },
    }
    _FORMATS = [
        *KuwoBaseIE._FORMATS,
        {'format': 'mkv', 'ext': 'mkv', 'preference': 250},
        {'format': 'mp4', 'ext': 'mp4', 'preference': 200}]

    def _real_extract(self, url):
        song_id = self._match_id(url)
        webpage = self._download_webpage(
            url, song_id, note=f'Download mv detail info: {song_id}',
            errnote=f'Unable to get mv detail info: {song_id}')

        mobj = re.search(
            r'<h1[^>]+title="(?P<song>[^"]+)">[^<]+<span[^>]+title="(?P<singer>[^"]+)"',
            webpage)
        if mobj:
            song_name = mobj.group('song')
            singer_name = mobj.group('singer')
        else:
            raise ExtractorError('Unable to find song or singer names')

        formats = self._get_formats(song_id, tolerate_ip_deny=True)

        mv_url = self._download_webpage(
            f'http://www.kuwo.cn/yy/st/mvurl?rid=MUSIC_{song_id}',
            song_id, note=f'Download {song_id} MV URL')
        formats.append({
            'url': mv_url,
            'format_id': 'mv',
        })

        return {
            'id': song_id,
            'title': song_name,
            'creator': singer_name,
            'formats': formats,
        }



import base64

from .common import InfoExtractor
from ..aes import aes_cbc_decrypt_bytes, unpad_pkcs7
from ..utils import (
    ExtractorError,
    unified_strdate,
)


class ShemarooMeIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?shemaroome\.com/(?:movies|shows)/(?P<id>[^?#]+)'
    _TESTS = [{
        'url': 'https://www.shemaroome.com/movies/dil-hai-tumhaara',
        'info_dict': {
            'id': 'dil-hai-tumhaara',
            'ext': 'mp4',
            'title': 'Dil Hai Tumhaara',
            'release_date': '20020906',
            'thumbnail': r're:^https?://.*\.jpg$',
            'description': 'md5:2782c4127807103cf5a6ae2ca33645ce',
        },
        'params': {
            'skip_download': True,
        },
    }, {
        'url': 'https://www.shemaroome.com/shows/jurm-aur-jazbaat/laalach',
        'info_dict': {
            'id': 'jurm-aur-jazbaat_laalach',
            'ext': 'mp4',
            'title': 'Laalach',
            'description': 'md5:92b79c2dcb539b0ab53f9fa5a048f53c',
            'thumbnail': r're:^https?://.*\.jpg$',
            'release_date': '20210507',
        },
        'params': {
            'skip_download': True,
        },
        'skip': 'Premium videos cannot be downloaded yet.',
    }, {
        'url': 'https://www.shemaroome.com/shows/jai-jai-jai-bajrang-bali/jai-jai-jai-bajrang-bali-episode-99',
        'info_dict': {
            'id': 'jai-jai-jai-bajrang-bali_jai-jai-jai-bajrang-bali-episode-99',
            'ext': 'mp4',
            'title': 'Jai Jai Jai Bajrang Bali Episode 99',
            'description': 'md5:850d127a18ee3f9529d7fbde2f49910d',
            'thumbnail': r're:^https?://.*\.jpg$',
            'release_date': '20110101',
        },
        'params': {
            'skip_download': True,
        },
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url).replace('/', '_')
        webpage = self._download_webpage(url, video_id)
        title = self._search_regex(r'id=\"ma_title\" value=\"([^\"]+)', webpage, 'title')
        thumbnail = self._og_search_thumbnail(webpage)
        content_def = self._search_regex(r'id=\"content_definition\" value=\"([^\"]+)', webpage, 'content_def')
        catalog_id = self._search_regex(r'id=\"catalog_id\" value=\"([^\"]+)', webpage, 'catalog_id')
        item_category = self._search_regex(r'id=\"item_category\" value=\"([^\"]+)', webpage, 'item_category')
        content_id = self._search_regex(r'id=\"content_id\" value=\"([^\"]+)', webpage, 'content_id')

        data = f'catalog_id={catalog_id}&content_id={content_id}&category={item_category}&content_def={content_def}'
        data_json = self._download_json('https://www.shemaroome.com/users/user_all_lists', video_id, data=data.encode())
        if not data_json.get('status'):
            raise ExtractorError('Premium videos cannot be downloaded yet.', expected=True)
        url_data = base64.b64decode(data_json['new_play_url'])
        key = base64.b64decode(data_json['key'])
        iv = bytes(16)
        m3u8_url = unpad_pkcs7(aes_cbc_decrypt_bytes(url_data, key, iv)).decode('ascii')
        headers = {'stream_key': data_json['stream_key']}
        formats, m3u8_subs = self._extract_m3u8_formats_and_subtitles(m3u8_url, video_id, fatal=False, headers=headers)
        for fmt in formats:
            fmt['http_headers'] = headers

        release_date = self._html_search_regex(
            (r'itemprop="uploadDate">\s*([\d-]+)', r'id="release_date" value="([\d-]+)'),
            webpage, 'release date', fatal=False)

        subtitles = {}
        sub_url = data_json.get('subtitle')
        if sub_url:
            subtitles.setdefault('EN', []).append({
                'url': self._proto_relative_url(sub_url),
            })
        subtitles = self._merge_subtitles(subtitles, m3u8_subs)
        description = self._html_search_regex(r'(?s)>Synopsis(</.+?)</', webpage, 'description', fatal=False)

        return {
            'id': video_id,
            'formats': formats,
            'title': title,
            'thumbnail': thumbnail,
            'release_date': unified_strdate(release_date),
            'description': description,
            'subtitles': subtitles,
        }



import re

from .common import InfoExtractor
from ..utils import (
    ExtractorError,
    int_or_none,
    parse_duration,
    traverse_obj,
    unified_timestamp,
    url_or_none,
    urljoin,
)


class RTVSLOIE(InfoExtractor):
    IE_NAME = 'rtvslo.si'
    _VALID_URL = r'''(?x)
        https?://(?:
            (?:365|4d)\.rtvslo.si/arhiv/[^/?#&;]+|
            (?:www\.)?rtvslo\.si/rtv365/arhiv
        )/(?P<id>\d+)'''
    _GEO_COUNTRIES = ['SI']

    _API_BASE = 'https://api.rtvslo.si/ava/{}/{}?client_id=82013fb3a531d5414f478747c1aca622'
    SUB_LANGS_MAP = {'Slovenski': 'sl'}

    _TESTS = [{
        'url': 'https://www.rtvslo.si/rtv365/arhiv/174842550?s=tv',
        'info_dict': {
            'id': '174842550',
            'ext': 'mp4',
            'release_timestamp': 1643140032,
            'upload_date': '20220125',
            'series': 'Dnevnik',
            'thumbnail': 'https://img.rtvcdn.si/_up/ava/ava_misc/show_logos/92/dnevnik_3_wide2.jpg',
            'description': 'md5:76a18692757aeb8f0f51221106277dd2',
            'timestamp': 1643137046,
            'title': 'Dnevnik',
            'series_id': '92',
            'release_date': '20220125',
            'duration': 1789,
        },
    }, {
        'url': 'https://365.rtvslo.si/arhiv/utrip/174843754',
        'info_dict': {
            'id': '174843754',
            'ext': 'mp4',
            'series_id': '94',
            'release_date': '20220129',
            'timestamp': 1643484455,
            'title': 'Utrip',
            'duration': 813,
            'thumbnail': 'https://img.rtvcdn.si/_up/ava/ava_misc/show_logos/94/utrip_1_wide2.jpg',
            'description': 'md5:77f2892630c7b17bb7a5bb84319020c9',
            'release_timestamp': 1643485825,
            'upload_date': '20220129',
            'series': 'Utrip',
        },
    }, {
        'url': 'https://365.rtvslo.si/arhiv/il-giornale-della-sera/174844609',
        'info_dict': {
            'id': '174844609',
            'ext': 'mp3',
            'series_id': '106615841',
            'title': 'Il giornale della sera',
            'duration': 1328,
            'series': 'Il giornale della sera',
            'timestamp': 1643743800,
            'release_timestamp': 1643745424,
            'thumbnail': 'https://img.rtvcdn.si/_up/ava/ava_misc/show_logos/il-giornale-della-sera_wide2.jpg',
            'upload_date': '20220201',
            'tbr': 128000,
            'release_date': '20220201',
        },
    }, {
        'url': 'https://365.rtvslo.si/arhiv/razred-zase/148350750',
        'info_dict': {
            'id': '148350750',
            'ext': 'mp4',
            'title': 'Prvi šolski dan, mozaična oddaja za mlade',
            'series': 'Razred zase',
            'series_id': '148185730',
            'duration': 1481,
            'upload_date': '20121019',
            'timestamp': 1350672122,
            'release_date': '20121019',
            'release_timestamp': 1350672122,
            'thumbnail': 'https://img.rtvcdn.si/_up/ava/ava_misc/show_logos/148185730/razred_zase_2014_logo_4d_wide2.jpg',
        },
    }, {
        'url': 'https://4d.rtvslo.si/arhiv/dnevnik/174842550',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        v_id = self._match_id(url)
        meta = self._download_json(self._API_BASE.format('getRecordingDrm', v_id), v_id)['response']

        thumbs = [{'id': k, 'url': v, 'http_headers': {'Accept': 'image/jpeg'}}
                  for k, v in (meta.get('images') or {}).items()]

        subs = {}
        for s in traverse_obj(meta, 'subs', 'subtitles', default=[]):
            lang = self.SUB_LANGS_MAP.get(s.get('language'), s.get('language') or 'und')
            subs.setdefault(lang, []).append({
                'url': s.get('file'),
                'ext': traverse_obj(s, 'format', expected_type=str.lower),
            })

        jwt = meta.get('jwt')
        if not jwt:
            raise ExtractorError('Site did not provide an authentication token, cannot proceed.')

        media = self._download_json(self._API_BASE.format('getMedia', v_id), v_id, query={'jwt': jwt})['response']

        formats = []
        skip_protocols = ['smil', 'f4m', 'dash']
        adaptive_url = traverse_obj(media, ('addaptiveMedia', 'hls_sec'), expected_type=url_or_none)
        if adaptive_url:
            formats = self._extract_wowza_formats(adaptive_url, v_id, skip_protocols=skip_protocols)

        adaptive_url = traverse_obj(media, ('addaptiveMedia_sl', 'hls_sec'), expected_type=url_or_none)
        if adaptive_url:
            for f in self._extract_wowza_formats(adaptive_url, v_id, skip_protocols=skip_protocols):
                formats.append({
                    **f,
                    'format_id': 'sign-' + f['format_id'],
                    'format_note': 'Sign language interpretation', 'preference': -10,
                    'language': (
                        'slv' if f.get('language') == 'eng' and f.get('acodec') != 'none'
                        else f.get('language')),
                })

        for mediafile in traverse_obj(media, ('mediaFiles', lambda _, v: url_or_none(v['streams']['https']))):
            formats.append(traverse_obj(mediafile, {
                'url': ('streams', 'https'),
                'ext': ('mediaType', {str.lower}),
                'width': ('width', {int_or_none}),
                'height': ('height', {int_or_none}),
                'tbr': ('bitrate', {int_or_none}),
                'filesize': ('filesize', {int_or_none}),
            }))

        for mediafile in traverse_obj(media, ('mediaFiles', lambda _, v: url_or_none(v['streams']['hls_sec']))):
            formats.extend(self._extract_wowza_formats(
                mediafile['streams']['hls_sec'], v_id, skip_protocols=skip_protocols))

        if any('intermission.mp4' in x['url'] for x in formats):
            self.raise_geo_restricted(countries=self._GEO_COUNTRIES, metadata_available=True)
        if any('dummy_720p.mp4' in x.get('manifest_url', '') for x in formats) and meta.get('stub') == 'error':
            raise ExtractorError(f'{self.IE_NAME} said: Clip not available', expected=True)

        return {
            'id': v_id,
            'webpage_url': ''.join(traverse_obj(meta, ('canonical', ('domain', 'path')))),
            'title': meta.get('title'),
            'formats': formats,
            'subtitles': subs,
            'thumbnails': thumbs,
            'description': meta.get('description'),
            'timestamp': unified_timestamp(traverse_obj(meta, 'broadcastDate', ('broadcastDates', 0))),
            'release_timestamp': unified_timestamp(meta.get('recordingDate')),
            'duration': meta.get('duration') or parse_duration(meta.get('length')),
            'tags': meta.get('genre'),
            'series': meta.get('showName'),
            'series_id': meta.get('showId'),
        }


class RTVSLOShowIE(InfoExtractor):
    IE_NAME = 'rtvslo.si:show'
    _VALID_URL = r'https?://(?:365|4d)\.rtvslo.si/oddaja/[^/?#&]+/(?P<id>\d+)'

    _TESTS = [{
        'url': 'https://365.rtvslo.si/oddaja/ekipa-bled/173250997',
        'info_dict': {
            'id': '173250997',
            'title': 'Ekipa Bled',
            'description': 'md5:c88471e27a1268c448747a5325319ab7',
            'thumbnail': 'https://img.rtvcdn.si/_up/ava/ava_misc/show_logos/173250997/logo_wide1.jpg',
        },
        'playlist_count': 18,
    }]

    def _real_extract(self, url):
        playlist_id = self._match_id(url)
        webpage = self._download_webpage(url, playlist_id)

        return self.playlist_from_matches(
            re.findall(r'<a [^>]*\bhref="(/arhiv/[^"]+)"', webpage),
            playlist_id, self._html_extract_title(webpage),
            getter=urljoin('https://365.rtvslo.si'), ie=RTVSLOIE,
            description=self._og_search_description(webpage),
            thumbnail=self._og_search_thumbnail(webpage),
        )



from .common import InfoExtractor
from ..utils import ExtractorError, traverse_obj


class IsraelNationalNewsIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?israelnationalnews\.com/news/(?P<id>\d+)'
    _TESTS = [{
        'url': 'https://www.israelnationalnews.com/news/354520',
        'info_dict': {
            'id': '354520',
        },
        'playlist': [{
            'info_dict': {
                'id': 'jA84wQhVvg8',
                'title': 'Even CNN Host Is Shocked by How Bad Biden\'s Approval Ratings Have Gotten | DM CLIPS | Rubin Report',
                'ext': 'mp4',
                'description': 'md5:b7325a3d00c7596337dc3ae37e32d35c',
                'channel': 'The Rubin Report',
                'channel_follower_count': int,
                'comment_count': int,
                'categories': ['News & Politics'],
                'like_count': int,
                'uploader_url': 'http://www.youtube.com/user/RubinReport',
                'uploader_id': 'RubinReport',
                'availability': 'public',
                'view_count': int,
                'duration': 240,
                'thumbnail': 'https://i.ytimg.com/vi_webp/jA84wQhVvg8/maxresdefault.webp',
                'live_status': 'not_live',
                'playable_in_embed': True,
                'age_limit': 0,
                'tags': 'count:29',
                'channel_id': 'UCJdKr0Bgd_5saZYqLCa9mng',
                'channel_url': 'https://www.youtube.com/channel/UCJdKr0Bgd_5saZYqLCa9mng',
                'upload_date': '20220606',
                'uploader': 'The Rubin Report',
            },
        }],
    }]

    def _real_extract(self, url):
        news_article_id = self._match_id(url)
        article_json = self._download_json(
            f'https://www.israelnationalnews.com/Generic/NewAPI/Item?type=0&Item={news_article_id}', news_article_id)

        urls = traverse_obj(article_json, ('Content2', ..., 'content', ..., 'attrs', 'src'))
        if not urls:
            raise ExtractorError('This article does not have any videos', expected=True)

        return self.playlist_from_matches(urls, news_article_id, ie='Youtube')



import re

from .common import InfoExtractor
from ..utils import (
    bool_or_none,
    int_or_none,
    parse_iso8601,
    traverse_obj,
    url_or_none,
)


class TV4IE(InfoExtractor):
    IE_DESC = 'tv4.se and tv4play.se'
    _VALID_URL = r'''(?x)https?://(?:www\.)?
        (?:
            tv4\.se/(?:[^/]+)/klipp/(?:.*)-|
            tv4play\.se/
            (?:
                (?:program|barn)/(?:(?:[^/]+/){1,2}|(?:[^\?]+)\?video_id=)|
                iframe/video/|
                film/|
                sport/|
            )
        )(?P<id>[0-9]+)'''
    _GEO_BYPASS = False
    _TESTS = [
        {
            # not geo-restricted
            'url': 'http://www.tv4.se/kalla-fakta/klipp/kalla-fakta-5-english-subtitles-2491650',
            'md5': 'cb837212f342d77cec06e6dad190e96d',
            'info_dict': {
                'id': '2491650',
                'ext': 'mp4',
                'title': 'Kalla Fakta 5 (english subtitles)',
                'description': '2491650',
                'series': 'Kalla fakta',
                'duration': 1335,
                'thumbnail': r're:^https?://[^/?#]+/api/v2/img/',
                'timestamp': 1385373240,
                'upload_date': '20131125',
            },
            'params': {'skip_download': 'm3u8'},
            'expected_warnings': ['Unable to download f4m manifest'],
        },
        {
            'url': 'http://www.tv4play.se/iframe/video/3054113',
            'md5': 'cb837212f342d77cec06e6dad190e96d',
            'info_dict': {
                'id': '3054113',
                'ext': 'mp4',
                'title': 'Så här jobbar ficktjuvarna - se avslöjande bilder',
                'thumbnail': r're:^https?://.*\.jpg$',
                'description': 'Unika bilder avslöjar hur turisternas fickor vittjas mitt på Stockholms central. Två experter på ficktjuvarna avslöjar knepen du ska se upp för.',
                'timestamp': int,
                'upload_date': '20150130',
            },
            'skip': '404 Not Found',
        },
        {
            'url': 'http://www.tv4play.se/sport/3060959',
            'only_matching': True,
        },
        {
            'url': 'http://www.tv4play.se/film/2378136',
            'only_matching': True,
        },
        {
            'url': 'http://www.tv4play.se/barn/looney-tunes?video_id=3062412',
            'only_matching': True,
        },
        {
            'url': 'http://www.tv4play.se/program/farang/3922081',
            'only_matching': True,
        },
        {
            'url': 'https://www.tv4play.se/program/nyheterna/avsnitt/13315940',
            'only_matching': True,
        },
    ]

    def _call_api(self, endpoint, video_id, headers=None, query={}):
        return self._download_json(
            f'https://playback2.a2d.tv/{endpoint}/{video_id}', video_id,
            f'Downloading {endpoint} API JSON', headers=headers, query={
                'service': 'tv4',
                'device': 'browser',
                'protocol': 'hls',
                **query,
            })

    def _real_extract(self, url):
        video_id = self._match_id(url)

        info = traverse_obj(self._call_api('asset', video_id, query={
            'protocol': 'hls,dash',
            'drm': 'widevine',
        }), ('metadata', {dict})) or {}

        manifest_url = self._call_api(
            'play', video_id, headers=self.geo_verification_headers())['playbackItem']['manifestUrl']

        formats, subtitles = [], {}

        fmts, subs = self._extract_m3u8_formats_and_subtitles(
            manifest_url, video_id, 'mp4',
            'm3u8_native', m3u8_id='hls', fatal=False)
        formats.extend(fmts)
        subtitles = self._merge_subtitles(subtitles, subs)

        fmts, subs = self._extract_mpd_formats_and_subtitles(
            manifest_url.replace('.m3u8', '.mpd'),
            video_id, mpd_id='dash', fatal=False)
        formats.extend(fmts)
        subtitles = self._merge_subtitles(subtitles, subs)

        fmts = self._extract_f4m_formats(
            manifest_url.replace('.m3u8', '.f4m'),
            video_id, f4m_id='hds', fatal=False)
        formats.extend(fmts)

        fmts, subs = self._extract_ism_formats_and_subtitles(
            re.sub(r'\.ism/.*?\.m3u8', r'.ism/Manifest', manifest_url),
            video_id, ism_id='mss', fatal=False)
        formats.extend(fmts)
        subtitles = self._merge_subtitles(subtitles, subs)

        if not formats and info.get('is_geo_restricted'):
            self.raise_geo_restricted(
                'This video is not available from your location due to geo-restriction, or not being authenticated',
                countries=['SE'])

        return {
            'id': video_id,
            'formats': formats,
            'subtitles': subtitles,
            **traverse_obj(info, {
                'title': ('title', {str}),
                'description': ('description', {str}),
                'timestamp': (('broadcast_date_time', 'broadcastDateTime'), {parse_iso8601}),
                'duration': ('duration', {int_or_none}),
                'thumbnail': ('image', {url_or_none}),
                'is_live': ('isLive', {bool_or_none}),
                'series': ('seriesTitle', {str}),
                'season_number': ('seasonNumber', {int_or_none}),
                'episode': ('episodeTitle', {str}),
                'episode_number': ('episodeNumber', {int_or_none}),
            }, get_all=False),
        }



import re
import urllib.parse

from .common import InfoExtractor
from ..networking.exceptions import HTTPError
from ..utils import (
    ExtractorError,
    determine_ext,
    extract_attributes,
    int_or_none,
    merge_dicts,
    parse_iso8601,
    strip_or_none,
    traverse_obj,
    url_or_none,
    urljoin,
)


class IGNBaseIE(InfoExtractor):
    def _call_api(self, slug):
        return self._download_json(
            f'http://apis.ign.com/{self._PAGE_TYPE}/v3/{self._PAGE_TYPE}s/slug/{slug}', slug)

    def _checked_call_api(self, slug):
        try:
            return self._call_api(slug)
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError) and e.cause.status == 404:
                e.cause.args = e.cause.args or [
                    e.cause.response.url, e.cause.status, e.cause.reason]
                raise ExtractorError(
                    'Content not found: expired?', cause=e.cause,
                    expected=True)
            raise

    def _extract_video_info(self, video, fatal=True):
        video_id = video['videoId']

        formats = []
        refs = traverse_obj(video, 'refs', expected_type=dict) or {}

        m3u8_url = url_or_none(refs.get('m3uUrl'))
        if m3u8_url:
            formats.extend(self._extract_m3u8_formats(
                m3u8_url, video_id, 'mp4', 'm3u8_native',
                m3u8_id='hls', fatal=False))

        f4m_url = url_or_none(refs.get('f4mUrl'))
        if f4m_url:
            formats.extend(self._extract_f4m_formats(
                f4m_url, video_id, f4m_id='hds', fatal=False))

        for asset in (video.get('assets') or []):
            asset_url = url_or_none(asset.get('url'))
            if not asset_url:
                continue
            formats.append({
                'url': asset_url,
                'tbr': int_or_none(asset.get('bitrate'), 1000),
                'fps': int_or_none(asset.get('frame_rate')),
                'height': int_or_none(asset.get('height')),
                'width': int_or_none(asset.get('width')),
            })

        mezzanine_url = traverse_obj(
            video, ('system', 'mezzanineUrl'), expected_type=url_or_none)
        if mezzanine_url:
            formats.append({
                'ext': determine_ext(mezzanine_url, 'mp4'),
                'format_id': 'mezzanine',
                'quality': 1,
                'url': mezzanine_url,
            })

        thumbnails = traverse_obj(
            video, ('thumbnails', ..., {'url': 'url'}), expected_type=url_or_none)
        tags = traverse_obj(
            video, ('tags', ..., 'displayName'),
            expected_type=lambda x: x.strip() or None)

        metadata = traverse_obj(video, 'metadata', expected_type=dict) or {}
        title = traverse_obj(
            metadata, 'longTitle', 'title', 'name',
            expected_type=lambda x: x.strip() or None)

        return {
            'id': video_id,
            'title': title,
            'description': strip_or_none(metadata.get('description')),
            'timestamp': parse_iso8601(metadata.get('publishDate')),
            'duration': int_or_none(metadata.get('duration')),
            'thumbnails': thumbnails,
            'formats': formats,
            'tags': tags,
        }


class IGNIE(IGNBaseIE):
    """
    Extractor for some of the IGN sites, like www.ign.com, es.ign.com de.ign.com.
    Some videos of it.ign.com are also supported
    """
    _VIDEO_PATH_RE = r'/(?:\d{4}/\d{2}/\d{2}/)?(?P<id>.+?)'
    _PLAYLIST_PATH_RE = r'(?:/?\?(?P<filt>[^&#]+))?'
    _VALID_URL = (
        r'https?://(?:.+?\.ign|www\.pcmag)\.com/videos(?:{})'.format('|'.join((_VIDEO_PATH_RE + r'(?:[/?&#]|$)', _PLAYLIST_PATH_RE))))
    IE_NAME = 'ign.com'
    _PAGE_TYPE = 'video'

    _TESTS = [{
        'url': 'http://www.ign.com/videos/2013/06/05/the-last-of-us-review',
        'md5': 'd2e1586d9987d40fad7867bf96a018ea',
        'info_dict': {
            'id': '8f862beef863986b2785559b9e1aa599',
            'ext': 'mp4',
            'title': 'The Last of Us Review',
            'description': 'md5:c8946d4260a4d43a00d5ae8ed998870c',
            'timestamp': 1370440800,
            'upload_date': '20130605',
            'tags': 'count:9',
            'display_id': 'the-last-of-us-review',
            'thumbnail': 'https://assets1.ignimgs.com/vid/thumbnails/user/2014/03/26/lastofusreviewmimig2.jpg',
            'duration': 440,
        },
        'params': {
            'nocheckcertificate': True,
        },
    }, {
        'url': 'http://www.pcmag.com/videos/2015/01/06/010615-whats-new-now-is-gogo-snooping-on-your-data',
        'md5': 'f1581a6fe8c5121be5b807684aeac3f6',
        'info_dict': {
            'id': 'ee10d774b508c9b8ec07e763b9125b91',
            'ext': 'mp4',
            'title': 'What\'s New Now: Is GoGo Snooping on Your Data?',
            'description': 'md5:817a20299de610bd56f13175386da6fa',
            'timestamp': 1420571160,
            'upload_date': '20150106',
            'tags': 'count:4',
        },
        'skip': '404 Not Found',
    }, {
        'url': 'https://www.ign.com/videos/is-a-resident-evil-4-remake-on-the-way-ign-daily-fix',
        'only_matching': True,
    }]

    @classmethod
    def _extract_embed_urls(cls, url, webpage):
        grids = re.findall(
            r'''(?s)<section\b[^>]+\bclass\s*=\s*['"](?:[\w-]+\s+)*?content-feed-grid(?!\B|-)[^>]+>(.+?)</section[^>]*>''',
            webpage)
        return filter(
            None, (urljoin(url, m.group('path')) for m in re.finditer(
                rf'''<a\b[^>]+\bhref\s*=\s*('|")(?P<path>/videos{cls._VIDEO_PATH_RE})\1''',
                grids[0] if grids else '')))

    def _real_extract(self, url):
        display_id, filt = self._match_valid_url(url).group('id', 'filt')
        if display_id:
            return self._extract_video(url, display_id)
        return self._extract_playlist(url, filt or 'all')

    def _extract_playlist(self, url, display_id):
        webpage = self._download_webpage(url, display_id)

        return self.playlist_result(
            (self.url_result(u, self.ie_key())
             for u in self._extract_embed_urls(url, webpage)),
            playlist_id=display_id)

    def _extract_video(self, url, display_id):
        video = self._checked_call_api(display_id)

        info = self._extract_video_info(video)

        return merge_dicts({
            'display_id': display_id,
        }, info)


class IGNVideoIE(IGNBaseIE):
    _VALID_URL = r'https?://.+?\.ign\.com/(?:[a-z]{2}/)?[^/]+/(?P<id>\d+)/(?:video|trailer)/'
    _TESTS = [{
        'url': 'http://me.ign.com/en/videos/112203/video/how-hitman-aims-to-be-different-than-every-other-s',
        'md5': 'dd9aca7ed2657c4e118d8b261e5e9de1',
        'info_dict': {
            'id': 'e9be7ea899a9bbfc0674accc22a36cc8',
            'ext': 'mp4',
            'title': 'How Hitman Aims to Be Different Than Every Other Stealth Game - NYCC 2015',
            'description': 'Taking out assassination targets in Hitman has never been more stylish.',
            'timestamp': 1444665600,
            'upload_date': '20151012',
            'display_id': '112203',
            'thumbnail': 'https://sm.ign.com/ign_me/video/h/how-hitman/how-hitman-aims-to-be-different-than-every-other-s_8z14.jpg',
            'duration': 298,
            'tags': 'count:13',
        },
        'expected_warnings': ['HTTP Error 400: Bad Request'],
    }, {
        'url': 'http://me.ign.com/ar/angry-birds-2/106533/video/lrd-ldyy-lwl-lfylm-angry-birds',
        'only_matching': True,
    }, {
        # Youtube embed
        'url': 'https://me.ign.com/ar/ratchet-clank-rift-apart/144327/trailer/embed',
        'only_matching': True,
    }, {
        # Twitter embed
        'url': 'http://adria.ign.com/sherlock-season-4/9687/trailer/embed',
        'only_matching': True,
    }, {
        # Vimeo embed
        'url': 'https://kr.ign.com/bic-2018/3307/trailer/embed',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        parsed_url = urllib.parse.urlparse(url)
        embed_url = urllib.parse.urlunparse(
            parsed_url._replace(path=parsed_url.path.rsplit('/', 1)[0] + '/embed'))

        webpage, urlh = self._download_webpage_handle(embed_url, video_id)
        new_url = urlh.url
        ign_url = urllib.parse.parse_qs(
            urllib.parse.urlparse(new_url).query).get('url', [None])[-1]
        if ign_url:
            return self.url_result(ign_url, IGNIE.ie_key())
        video = self._search_regex(r'(<div\b[^>]+\bdata-video-id\s*=\s*[^>]+>)', webpage, 'video element', fatal=False)
        if not video:
            if new_url == url:
                raise ExtractorError('Redirect loop: ' + url)
            return self.url_result(new_url)
        video = extract_attributes(video)
        video_data = video.get('data-settings') or '{}'
        video_data = self._parse_json(video_data, video_id)['video']
        info = self._extract_video_info(video_data)

        return merge_dicts({
            'display_id': video_id,
        }, info)


class IGNArticleIE(IGNBaseIE):
    _VALID_URL = r'https?://.+?\.ign\.com/(?:articles(?:/\d{4}/\d{2}/\d{2})?|(?:[a-z]{2}/)?(?:[\w-]+/)*?feature/\d+)/(?P<id>[^/?&#]+)'
    _PAGE_TYPE = 'article'
    _TESTS = [{
        'url': 'http://me.ign.com/en/feature/15775/100-little-things-in-gta-5-that-will-blow-your-mind',
        'info_dict': {
            'id': '72113',
            'title': '100 Little Things in GTA 5 That Will Blow Your Mind',
        },
        'playlist': [
            {
                'info_dict': {
                    'id': '5ebbd138523268b93c9141af17bec937',
                    'ext': 'mp4',
                    'title': 'Grand Theft Auto V Video Review',
                    'description': 'Rockstar drops the mic on this generation of games. Watch our review of the masterly Grand Theft Auto V.',
                    'timestamp': 1379339880,
                    'upload_date': '20130916',
                    'tags': 'count:12',
                    'thumbnail': 'https://assets1.ignimgs.com/thumbs/userUploaded/2021/8/16/gta-v-heistsjpg-e94705-1629138553533.jpeg',
                    'display_id': 'grand-theft-auto-v-video-review',
                    'duration': 501,
                },
            },
            {
                'info_dict': {
                    'id': '638672ee848ae4ff108df2a296418ee2',
                    'ext': 'mp4',
                    'title': 'GTA 5 In Slow Motion',
                    'description': 'The twisted beauty of GTA 5 in stunning slow motion.',
                    'timestamp': 1386878820,
                    'upload_date': '20131212',
                    'duration': 202,
                    'tags': 'count:25',
                    'display_id': 'gta-5-in-slow-motion',
                    'thumbnail': 'https://assets1.ignimgs.com/vid/thumbnails/user/2013/11/03/GTA-SLO-MO-1.jpg',
                },
            },
        ],
        'params': {
            'skip_download': True,
        },
        'expected_warnings': ['Backend fetch failed'],
    }, {
        'url': 'http://www.ign.com/articles/2014/08/15/rewind-theater-wild-trailer-gamescom-2014?watch',
        'info_dict': {
            'id': '53ee806780a81ec46e0790f8',
            'title': 'Rewind Theater - Wild Trailer Gamescom 2014',
        },
        'playlist_count': 1,
        'expected_warnings': ['Backend fetch failed'],
    }, {
        # videoId pattern
        'url': 'http://www.ign.com/articles/2017/06/08/new-ducktales-short-donalds-birthday-doesnt-go-as-planned',
        'only_matching': True,
    }, {
        # Youtube embed
        'url': 'https://www.ign.com/articles/2021-mvp-named-in-puppy-bowl-xvii',
        'only_matching': True,
    }, {
        # IMDB embed
        'url': 'https://www.ign.com/articles/2014/08/07/sons-of-anarchy-final-season-trailer',
        'only_matching': True,
    }, {
        # Facebook embed
        'url': 'https://www.ign.com/articles/2017/09/20/marvels-the-punisher-watch-the-new-trailer-for-the-netflix-series',
        'only_matching': True,
    }, {
        # Brightcove embed
        'url': 'https://www.ign.com/articles/2016/01/16/supergirl-goes-flying-with-martian-manhunter-in-new-clip',
        'only_matching': True,
    }]

    def _checked_call_api(self, slug):
        try:
            return self._call_api(slug)
        except ExtractorError as e:
            if isinstance(e.cause, HTTPError):
                e.cause.args = e.cause.args or [
                    e.cause.response.url, e.cause.status, e.cause.reason]
                if e.cause.status == 404:
                    raise ExtractorError(
                        'Content not found: expired?', cause=e.cause,
                        expected=True)
                elif e.cause.status == 503:
                    self.report_warning(str(e.cause))
                    return
            raise

    def _real_extract(self, url):
        display_id = self._match_id(url)
        article = self._checked_call_api(display_id)

        if article:
            # obsolete ?
            def entries():
                media_url = traverse_obj(
                    article, ('mediaRelations', 0, 'media', 'metadata', 'url'),
                    expected_type=url_or_none)
                if media_url:
                    yield self.url_result(media_url, IGNIE.ie_key())
                for content in (article.get('content') or []):
                    for video_url in re.findall(r'(?:\[(?:ignvideo\s+url|youtube\s+clip_id)|<iframe[^>]+src)="([^"]+)"', content):
                        if url_or_none(video_url):
                            yield self.url_result(video_url)

            return self.playlist_result(
                entries(), article.get('articleId'),
                traverse_obj(
                    article, ('metadata', 'headline'),
                    expected_type=lambda x: x.strip() or None))

        webpage = self._download_webpage(url, display_id)

        playlist_id = self._html_search_meta('dable:item_id', webpage, default=None)
        if playlist_id:

            def entries():
                for m in re.finditer(
                        r'''(?s)<object\b[^>]+\bclass\s*=\s*("|')ign-videoplayer\1[^>]*>(?P<params>.+?)</object''',
                        webpage):
                    flashvars = self._search_regex(
                        r'''(<param\b[^>]+\bname\s*=\s*("|')flashvars\2[^>]*>)''',
                        m.group('params'), 'flashvars', default='')
                    flashvars = urllib.parse.parse_qs(extract_attributes(flashvars).get('value') or '')
                    v_url = url_or_none((flashvars.get('url') or [None])[-1])
                    if v_url:
                        yield self.url_result(v_url)
        else:
            playlist_id = self._search_regex(
                r'''\bdata-post-id\s*=\s*("|')(?P<id>[\da-f]+)\1''',
                webpage, 'id', group='id', default=None)

            nextjs_data = self._search_nextjs_data(webpage, display_id)

            def entries():
                for player in traverse_obj(
                        nextjs_data,
                        ('props', 'apolloState', 'ROOT_QUERY', lambda k, _: k.startswith('videoPlayerProps('), '__ref')):
                    # skip promo links (which may not always be served, eg GH CI servers)
                    if traverse_obj(nextjs_data,
                                    ('props', 'apolloState', player.replace('PlayerProps', 'ModernContent')),
                                    expected_type=dict):
                        continue
                    video = traverse_obj(nextjs_data, ('props', 'apolloState', player), expected_type=dict) or {}
                    info = self._extract_video_info(video, fatal=False)
                    if info:
                        yield merge_dicts({
                            'display_id': display_id,
                        }, info)

        return self.playlist_result(
            entries(), playlist_id or display_id,
            re.sub(r'\s+-\s+IGN\s*$', '', self._og_search_title(webpage, default='')) or None)
